using System;
using System.Collections;
using System.Web.UI;
using System.Web.UI.Design;
using System.Web.UI.WebControls;
using System.ComponentModel;
using System.ComponentModel.Design;
using RepPP.Toolkit;
using RepPP;

namespace RepPP.Toolkit.Web {
	/// <summary>
	/// Summary description for WebFldValidator.
	/// </summary>
	[ToolboxData("<{0}:FldValidator runat=server></{0}:FldValidator>")]
	public class FldValidator : BaseValidator {
	    
	    /// <summary>Auto position of the validator</summary>
	    public enum AutoPos {
	        /// <summary>No auto positionning</summary>
	        None    =   0,
	        /// <summary>Before the validated field</summary>
	        Before  =   1,
	        /// <summary>After the validated field</summary>
	        After   =   2
	    };
	    
	    private const string            m_strFuncPrefix = "RepPP_ValidateField_";
	    private const string            m_strValidator  = "RepPPValidatorEvaluateIsValid";
	    private bool                    m_bDisposed;
	    /// <summary>Control utility class</summary>
	    protected ControlUtil           m_controlUtil;
	    /// <summary>Father control if embedded</summary>
	    protected WebControl            m_ctlFather;

        //********************************************************************************
        //
		/// <summary>
		/// ctor:               Class constructor
		/// </summary>
		//
        //********************************************************************************
		public FldValidator(WebControl ctlFather) {
		    m_controlUtil = new ControlUtil(this, true);
		    m_ctlFather   = ctlFather;
		    m_bDisposed   = false;
            Display       = ValidatorDisplay.Dynamic;
		    if (m_ctlFather != null) {
		        m_ctlFather.DataBinding += new EventHandler(m_ctlFather_DataBinding);
		        m_ctlFather.Init        += new EventHandler(m_ctlFather_Init);
		        m_ctlFather.Load        += new EventHandler(m_ctlFather_Load);
		        m_ctlFather.PreRender   += new EventHandler(m_ctlFather_PreRender);
		        m_ctlFather.Unload      += new EventHandler(m_ctlFather_Unload);
		    }
		}

        //********************************************************************************
        //
		/// <summary>
		/// ctor:               Class constructor
		/// </summary>
		//
        //********************************************************************************
		public FldValidator() : this(null) {
		}

        //********************************************************************************
        //
		/// <summary>
		/// Dispose:            Dispose the control
		/// </summary>
		//
        //********************************************************************************
        public override void Dispose() {
            if (!m_bDisposed) {
                m_bDisposed = true;
                if (m_ctlFather != null) {
		            m_ctlFather.DataBinding -= new EventHandler(m_ctlFather_DataBinding);
		            m_ctlFather.Init        -= new EventHandler(m_ctlFather_Init);
		            m_ctlFather.Load        -= new EventHandler(m_ctlFather_Load);
		            m_ctlFather.PreRender   -= new EventHandler(m_ctlFather_PreRender);
		            m_ctlFather.Unload      -= new EventHandler(m_ctlFather_Unload);
		        }
                if (m_controlUtil != null) {
                    m_controlUtil.Dispose();
                }
                base.Dispose();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// m_ctlFather_Load:       Called when the father is load
        /// </summary>
        /// <param name="sender">   Sender object</param>
        /// <param name="e">        Event parameters</param>
        //
        //********************************************************************************
        private void m_ctlFather_Load(object sender, EventArgs e) {
            OnLoad(e);
        }

        //********************************************************************************
        //
        /// <summary>
        /// m_ctlFather_DataBinding:Called when the databinding is in progress
        /// </summary>
        /// <param name="sender">   Sender object</param>
        /// <param name="e">        Event parameters</param>
        //
        //********************************************************************************
        private void m_ctlFather_DataBinding(object sender, EventArgs e) {
            OnDataBinding(e);
        }

        //********************************************************************************
        //
        /// <summary>
        /// m_ctlFather_Init:       Called when the father is initialized
        /// </summary>
        /// <param name="sender">   Sender object</param>
        /// <param name="e">        Event parameters</param>
        //
        //********************************************************************************
        private void m_ctlFather_Init(object sender, EventArgs e) {
            Page            = m_ctlFather.Page;
            OnInit(e);
        }

        //********************************************************************************
        //
        /// <summary>
        /// m_ctlFather_PreRender:  Called before the father is rendered
        /// </summary>
        /// <param name="sender">   Sender object</param>
        /// <param name="e">        Event parameters</param>
        //
        //********************************************************************************
        private void m_ctlFather_PreRender(object sender, EventArgs e) {
            OnPreRender(e);
        }

        //********************************************************************************
        //
        /// <summary>
        /// m_ctlFather_Unload:     Called when the father is unloaf
        /// </summary>
        /// <param name="sender">   Sender object</param>
        /// <param name="e">        Event parameters</param>
        //
        //********************************************************************************
        private void m_ctlFather_Unload(object sender, EventArgs e) {
            OnUnload(e);
        }

        //********************************************************************************
        //
        /// <summary>
        /// TriggerComponentChange:     Trigger the event indicating a component property has changed
        /// </summary>
        //
        //********************************************************************************
        public virtual void TriggerComponentChange() {
            if (m_controlUtil != null) {
                m_controlUtil.TriggerComponentChange();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Application:            Gets the Rep++ application
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public Application Application {
		    get {
		        return(m_controlUtil.Application);
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// FieldFullName:          Gets the field full name from the attached control
        /// </summary>
        //
        //********************************************************************************
		private FldFullName FieldFullName {
		    get {
		        FldFullName     fldFullNameRetVal = null;
		        string          strCtlName;
		        Control         ctl;
		        
		        strCtlName = ControlToValidate;
		        if (strCtlName != null && strCtlName.Length != 0) {
                    ctl = NamingContainer.FindControl(strCtlName);
                    if (ctl != null && ctl is FldControlUtil.IFldControl) {
                        fldFullNameRetVal = ((FldControlUtil.IFldControl)ctl).FieldFullName;
                        if (fldFullNameRetVal != null) {
                            if (fldFullNameRetVal.GroupName == null         ||
                                fldFullNameRetVal.GroupName.Length == 0     ||
                                fldFullNameRetVal.FieldName == null         ||
                                fldFullNameRetVal.FieldName.Length == 0) {
                                fldFullNameRetVal = null;
                            }
                        }
                    }
                }
                return(fldFullNameRetVal);
            }
		}

        //********************************************************************************
        //
		/// <summary>
		/// Field:                  Gets the Rep++ field from the attached control
		/// </summary>
		//
        //********************************************************************************
		private Field Field {
		    get {
		        Field       fldRetVal = null;
		        FldFullName fieldFullName;

                fieldFullName = FieldFullName;
                if (fieldFullName != null) {
                    fldRetVal = Application.Field(fieldFullName.GroupName, fieldFullName.FieldName);
                }
                return(fldRetVal);
		    }
		}

        //********************************************************************************
        //
		/// <summary>
		/// NamingContainer:        Return the father naming container if the control is embedded
		/// </summary>
		//
        //********************************************************************************
        public override Control NamingContainer {
            get {
                Control ctlRetVal;
                
                ctlRetVal = (m_ctlFather != null) ? m_ctlFather.NamingContainer : base.NamingContainer;
                return(ctlRetVal);
            }
        }

        //********************************************************************************
        //
		/// <summary>
		/// AutoPosition:       Type of positionning
		/// </summary>
		//
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public AutoPos AutoPosition {
		    get {
                AutoPos eRetVal;
                object  objAuto;
                
                objAuto = ViewState["AutoPosition"];
                if (objAuto != null) {
                    eRetVal = (AutoPos)(int)objAuto;
                } else {
                    eRetVal = AutoPos.None;
                }
                return(eRetVal);
            }
            set {
                ViewState["AutoPosition"] = (int)value;
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// AddAttributesToRender:  Adds attribute to render
        /// </summary>
        /// <param name="writer">   HTML text writer</param>
        //
        //********************************************************************************
        protected override void AddAttributesToRender(HtmlTextWriter writer) {
            FldFullName     fieldFullName;
            
            base.AddAttributesToRender(writer);
            if (base.RenderUplevel) {
                writer.AddAttribute("evaluationfunction", m_strValidator);
                fieldFullName = FieldFullName;
                if (fieldFullName != null) {
                    writer.AddAttribute("clientvalidationfunction", 
                                        m_strFuncPrefix + fieldFullName.GroupName + "_" + fieldFullName.FieldName);
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// EvaluateIsValid:        Determine if the current value is valid
        /// </summary>
        /// <returns>
        /// true if yes, false if not
        ///</returns>
        //
        //********************************************************************************
        protected override bool EvaluateIsValid() {
            bool    bRetVal;
            Field   fld;
            
            fld = Field;
            if (fld == null) {
                bRetVal = true;
            } else {
                fld.Application.FldOnError += new FldOnErrorEventHandler(Application_FldOnError);
                bRetVal = (fld.Validate() == 0);
                fld.Application.FldOnError -= new FldOnErrorEventHandler(Application_FldOnError);
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// InitFromFather:         Initialize the validator from the father web control
        /// </summary>
        /// <param name="ctlFather">Father Web Control</param>
        //
        //********************************************************************************
        public void InitFromFather(WebControl ctlFather) {
            ID               = "validate" + ctlFather.ID;
            EnableViewState  = false;
            //ErrorMessage     = "";
            //Text             = "";
            if (ctlFather.Style["Top"] != null) {
                Style["Position"]= "Absolute";
                Style["Top"]     = ctlFather.Style["Top"];
                Style["Left"]    = ctlFather.Style["Left"];
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// ControlToValidate:      Control to validate
        /// </summary>
        //
        //********************************************************************************
        [TypeConverter(typeof(FldValidatedControlConverter))]
        [Category("Behavior")]
        [DefaultValue("")]
        [Description("Control to validate")]
        public new string ControlToValidate {
            get {
                return(base.ControlToValidate);
            }
            set {
                base.ControlToValidate = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// GetPrefix:          Gets the control prefix in the HTML document
        /// </summary>
        /// <param name="ctl">  Control</param>
        /// <returns>
        /// Prefix
        /// </returns>
        //
        //********************************************************************************
        private string GetPrefix(Control ctl) {
            string  strRetVal;
            
            strRetVal = "";
            foreach (Control ctlChild in ctl.Page.Controls) {
                if (ctlChild is System.Web.UI.HtmlControls.HtmlForm) {
                    strRetVal = ((System.Web.UI.HtmlControls.HtmlForm)ctlChild).Name + ".";
                    break;
                }
            }
            return(strRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// AddJSValidationCode:    Adds the Javascript validation code to the page
        /// </summary>
        //
        //********************************************************************************
        private void AddJSValidationCode() {
            Field       fld;
            IntCodeGen  intCodeGen;
            string      strCode;
            string      strKey;
            string      strFuncName;
            string      strFldName;
            string      strAtomText;
            string      strControl;

            fld = Field;
            if (fld != null) {
                strFldName  = fld.Group.Name + "_" + fld.Name;
                if (!WebUtil.IsClientScriptBlockRegistered(Page, m_strValidator)) {
                    strAtomText = (fld.Application.IsAtomExist("Err_Error", AtomType.sdAtomErr)) ?
                                   fld.Application.GetAtomText("Err_Error", AtomType.sdAtomErr) : "";
                    strCode =   "<script language=\"JavaScript\">\r\n" +
                                "<!-- \r\n" +
                                "var RepPP_LastErrorMessage;\r\n\r\n" +
                                "var RepPP_LastErrorText;\r\n\r\n" +
                                "function RepPP_DisplayErrorOverride(sPrompt, arrErr) {\r\n" +
                                "     var sErrorMes;\r\n" +
                                "\r\n" +
                                "    sErrorMes = \"\";\r\n" +
                                "    for (iError in arrErr) {\r\n"+
                                "        sErrorMes = sErrorMes + ((sErrorMes.length == 0) ? \"\" : \",\") + arrErr[iError];\r\n" +
                                "    }\r\n" +
                                "    RepPP_LastErrorMessage = \"" + strAtomText + " \" + sPrompt + \" - \" + sErrorMes;\r\n" +
                                "    RepPP_LastErrorText    = sErrorMes;\r\n" +
                                "}\r\n" +
                                "function " + m_strValidator + "(val) {\r\n" +
                                "    var value = ValidatorGetValue(val.attributes[\"controltovalidate\"].value);\r\n" +
                                "    var args = { Value:value, IsValid:true };\r\n" +
                                "    if (typeof(val.attributes[\"clientvalidationfunction\"].value) == \"string\") {\r\n" +
                                "        eval(val.attributes[\"clientvalidationfunction\"].value + \"(val, args) ;\");\r\n" +
                                "    }\r\n" +
                                "    return args.IsValid;\r\n" +
                                "}\r\n" +
                                " -->\r\n" +
                                "</script>\r\n";
                    WebUtil.RegisterClientScriptBlock(Page, m_strValidator, strCode);                                
                }                         
                strKey      = "RepPPValidationCode_" + fld.Group.Name;
                if (!WebUtil.IsClientScriptBlockRegistered(Page, strKey)) {
                    intCodeGen = fld.Application.CreateExtObj(ExtObject.sdIntCodeGen) as IntCodeGen;
                    if (intCodeGen != null) {
                        foreach (Field fldInGrp in fld.Group.Fields) {
                            intCodeGen.AddFieldObjToCodeGeneration(fldInGrp, fldInGrp.Group.Name, fldInGrp.Name);
                        }
                        strCode = intCodeGen.GenerateCode(HTMLScriptType.sdHTMLJScript, HTMLGenOption.sdHTMLGenNone);
                        strCode = strCode + "\r\n";
                        WebUtil.RegisterClientScriptBlock(Page, strKey, strCode);
                    }
                }
                strFuncName = m_strFuncPrefix + strFldName;
                strControl   = "document.getElementById('" + m_ctlFather.ClientID +"')";
                strCode     = "<script language=\"Javascript\">\r\n" +
                              "<!--\r\n" +
                              "function " + strFuncName + "(val, args) {\r\n" +
                              "\r\n" +
                              "    args.IsValid = RepPP_Valid_" + strFldName + "(document.getElementById(val.attributes[\"controltovalidate\"].value), args.Value);\r\n" +
                              "    if (!args.IsValid) {\r\n" +
                              "        val.innerHTML    = RepPP_LastErrorText;\r\n" +
                              "        val.errormessage = RepPP_LastErrorMessage;\r\n";
                if (AutoPosition != AutoPos.None) {
                    strCode = strCode +
                              "        val.style.top = " + strControl + ".style.top;\r\n" +
                              "        val.style.position = " + strControl + ".style.position;\r\n";
                    if (AutoPosition == AutoPos.After) {
                        strCode = strCode + 
                                  "        if (" + strControl + ".style.left != \"\") {\r\n" +
                                  "            val.style.left = new Number(" + strControl + ".style." +
                                  "left.replace(\"px\",\"\")) + " + strControl + ".offsetWidth + 5 + \"px\";\r\n" +
                                  "        }\r\n";
                    } else {
                        strCode = strCode +
                                "        val.style.left = " + strControl + ".style.left;\r\n" +
                                "        " + strControl + ".style.left = new Number(val.style.left.replace(\"px\",\"\")) + " +
                                "val.offsetWidth + 5 + \"px\";\r\n" +
                                "    } else {\r\n" +
                                "        if (val.style.left != \"\") {\r\n" +
                                "            " + strControl + ".style.left = val.style.left;\r\n" +
                                "        }\r\n";
                    }
                }
                strCode     = strCode +                              
                              "    }\r\n" +
                              "}\r\n" +
                              " -->\r\n" +
                              "</script>\r\n";
                WebUtil.RegisterClientScriptBlock(Page, strFuncName, strCode);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// SetFieldError:          Set the error text for a validation
        /// </summary>
        /// <param name="fld">      Field in error</param>
        /// <param name="strError"> Error text </param>
        //
        //********************************************************************************
        virtual protected void SetFieldError(Field fld, string strError) {
            Text            = strError;
            ErrorMessage    = fld.Prompt + " - " + strError;
        }

        //********************************************************************************
        //
        /// <summary>
        /// Application_FldOnError: Event send when a field is in error
        /// </summary>
        /// <param name="sender">   Sender object (Rep++ application)</param>
        /// <param name="e">        Event parameter</param>
        //
        //********************************************************************************
        private void Application_FldOnError(object sender, FieldGenEventArgs e) {
            SetFieldError(e.Field, e.ErrorText);
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnPreRender:        Called before rendering is done
        /// </summary>
        /// <param name="e">    Event parameters</param>
        //
        //********************************************************************************
        protected override void OnPreRender(EventArgs e) {
            string strScript;

            base.OnPreRender(e);
            RenderJSValidation();
            if (!WebUtil.IsClientScriptBlockRegistered(Page, "ForceValidatorIncludeScript")&&
                !Page.Request.Browser.Browser.ToUpper().Equals("IE")) {
                strScript = "\r\n<script language=\"javascript\" type=\"text/javascript\">\r\nValidatorOnLoad();\r\n</script>";
                WebUtil.RegisterStartupScript(Page, "ForceValidatorIncludeScript", strScript);
            }            
        }

        //********************************************************************************
        //
        /// <summary>
        /// RenderJSValidation:        Render the javascript validation code
        /// </summary>
        //
        //********************************************************************************
        public void RenderJSValidation() {
            if (DetermineRenderUplevel()) {
                AddJSValidationCode();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// RegisterValidatorDeclaration:        Standardizes the emitted js
        /// </summary>
        //
        //********************************************************************************
        protected override void RegisterValidatorDeclaration() {
            WebUtil.RegisterArrayDeclaration(Page, "Page_Validators", "document.getElementById(\"" + this.ClientID + "\")");
        }

        //********************************************************************************
        //
        /// <summary>
        /// Type converter for the attached control. Implement a list of standard values for
        /// this property.
        /// </summary>
        //
        //********************************************************************************
        public class FldValidatedControlConverter : System.ComponentModel.StringConverter {

            //********************************************************************************
            //
            /// <summary>
            /// GetControls:                Gets the list of control which we can attached to
            /// </summary>
            /// <param name="container">    Container</param>
            /// <returns>
            /// List of controls name
            /// </returns>
            //
            //********************************************************************************
            private object[] GetControls(IContainer container) {
                ArrayList                   listRetVal;
                ComponentCollection         colComponents;
                Control                     ctl;
                ValidationPropertyAttribute attr;
            
                listRetVal      =  new ArrayList();
                colComponents   = container.Components;
                foreach (IComponent comp in colComponents) {
                    if (comp is Control && comp is FldControlUtil.IFldControl) {
                        ctl = (Control)comp;
                        if (ctl.ID != null && ctl.ID.Length != 0) {
                            attr = (ValidationPropertyAttribute)TypeDescriptor.GetAttributes(ctl)[typeof(ValidationPropertyAttribute)];
                            if (attr != null && attr.Name != null) {
                                listRetVal.Add(string.Copy(ctl.ID));
                            }
                        }
                    }
                }
                listRetVal.Sort(Comparer.Default);
                return(listRetVal.ToArray());
            }

            //********************************************************************************
            //
            /// <summary>
            /// GetStandardValues:  Gets the standard values of the type
            /// </summary>
            /// <param name="ctx"></param>
            /// <returns>
            /// List of values
            /// </returns>
            //
            //********************************************************************************
            public override TypeConverter.StandardValuesCollection GetStandardValues(ITypeDescriptorContext ctx) {
                TypeConverter.StandardValuesCollection  colRetVal = null;
                object[]                                objArray;
                
                if (ctx != null && ctx.Container != null) {
                    objArray = GetControls(ctx.Container);
                    if (objArray != null) {
                        colRetVal = new TypeConverter.StandardValuesCollection(objArray);
                    }
                }
                return(colRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// GetStandardValuesExclusive: Returns whether the collection of standard values returned from GetStandardValues is an exclusive list.
            /// </summary>
            /// <param name="ctx">          Context</param>
            /// <returns>
            /// false
            /// </returns>
            //
            //********************************************************************************
            public override bool GetStandardValuesExclusive(ITypeDescriptorContext ctx) {
                return(false);
            }

            //********************************************************************************
            //
            /// <summary>
            /// GetStandardValuesSupported: Returns true to indicate we support standard values
            /// </summary>
            /// <param name="ctx">          Context</param>
            /// <returns>
            /// true
            /// </returns>
            //
            //********************************************************************************
            public override bool GetStandardValuesSupported(ITypeDescriptorContext ctx) {
                return(true);
            }
        } // Class FldValidatedControlConverter
    } // Class FldValidator
} // Namespace
