using System;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

namespace RepPP.Toolkit.Web {
    //********************************************************************************
    //
	/// <summary>
	/// TemplateFldControl:    Implement an ITemplate interface to add a field web control
	///                        to a container. this class can be used to add a template column
	///                        to a datagrid or a datalist.
	/// </summary>
    //
    //********************************************************************************
	public class TemplateFldControl : ITemplate {
        /// <summary>Attached Field </summary>
        private RepPP.Field                     m_fld;
        /// <summary>Field web control type </summary>
        private WebUtil.FieldWebControlTypeE    m_eFldCtlType;

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                           Class constructor
        /// </summary>
        /// <param name="fld">              Field</param>
        /// <param name="eFldCtlType">      Type of the control to be instanciated</param>
	    //
        //********************************************************************************
		public TemplateFldControl(RepPP.Field fld, WebUtil.FieldWebControlTypeE eFldCtlType) {
            m_fld         = fld;
            m_eFldCtlType = eFldCtlType;
		}

        //********************************************************************************
        //
        /// <summary>
        /// FieldName:                      Get the field name
        /// </summary>
	    //
        //********************************************************************************
        public string FieldName {
            get {
                return(m_fld.Name);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// InstantiateIn:      Instanciate the attached control in the container
        /// </summary>
	    //
        //********************************************************************************
        void ITemplate.InstantiateIn(System.Web.UI.Control container) {
            Control ctlFld;

            ctlFld = WebUtil.CreateFldControl(m_eFldCtlType);
            (ctlFld as FldControlUtil.IFldControl).FieldFullName = new FldFullName(m_fld.Group.Name, m_fld.Name);
            if (ctlFld is IFldEditableControl) {
                (ctlFld as IFldEditableControl).ValidatorPosition = FldValidator.AutoPos.None;
            }
            ctlFld.DataBinding += new EventHandler(OnCtlDataBinding);
            container.Controls.Add(ctlFld);
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnCtlDataBinding:       Handle the control databinding event
        /// </summary>
        /// <param name="sender">   Sender object</param>
        /// <param name="e">        Event arguments</param>
	    //
        //********************************************************************************
        protected virtual void OnCtlDataBinding(object sender, EventArgs e) {
            Control         ctl;
            DataGridItem    container;
            DataRowGInst    row;

            ctl         = (Control)sender;
            container   = (DataGridItem)ctl.NamingContainer;
            row         = container.DataItem as DataRowGInst;
            if (row != null && ctl is IFldValueControl) {
                (ctl as IFldValueControl).AttachedLine = row.GroupInstance.LineFromBookmark(row.Bookmark);
            }
        }
    } // Class TemplateFldControl
} // Namespace
