//*********************************************************
//
// WndCheckIntegrityUI:    Form for base Integrity checking
//
// Author:	               Consyst SQL inc.    
//
// (c) 2005 by Consyst SQL Inc.
//
//*********************************************************
using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;

namespace  RepPP.Toolkit.Window {
	/// <summary>
	/// CheckIntegrityUI: Default user interface for CheckIntegrity class
	/// </summary>
	public class CheckIntegrityUI : System.Windows.Forms.Form {
        /// <summary>Listbox Parameters</summary>
        private const int                       HEIGHT_LISTBOX       = 160;
        private const int                       MARGIN_LISTBOX       = 8;
        private const int                       HEIGHTLABEL_LISTBOX  = 16;
        /// <summary>Integrity Manager</summary>
        protected RepPP.Toolkit.CheckIntegrity  m_checkIntegrity;
        /// <summary>List boxes created</summary>
        protected ArrayList                     m_arrLstBoxIntegrity;
        /// <summary>Delegate for OnIntegrityClick event</summary>
        public    delegate void                 OnIntegrityClickHandler(object sender, IntegrityEventArgs e);
        /// <summary>Occurs when an item from the integrities' list is clicking</summary>
        public    event		                    OnIntegrityClickHandler IntegrityClick;
        /// <summary>Occurs when an item from the integrities' list is double-clicking</summary>
        public    event		                    OnIntegrityClickHandler IntegrityDoubleClick;
        /// <summary>Panel Top where message is writted</summary>
        protected System.Windows.Forms.Panel    pnlTop;
        /// <summary>Integrity Message</summary>
        protected System.Windows.Forms.Label    lblIntegMessage;
        /// <summary>Main Panel where the integrities are listed</summary>
        protected System.Windows.Forms.Panel    pnlMain;
        /// <summary>Panel Bottom</summary>
        protected System.Windows.Forms.Panel    pnlBottom;
        /// <summary>Button</summary>
        protected System.Windows.Forms.Button   btnOK;
        /// <summary>Rep++ Object Application</summary>
        protected RepPP.Application             m_app;

        //******************************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        //
        //******************************************************************
		public CheckIntegrityUI(){
			InitializeComponent();
		}

        //******************************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        /// <param name="connection">   Data connection</param>
        //
        //******************************************************************
		public CheckIntegrityUI(RepPP.Connection  connection){
            m_checkIntegrity     = new RepPP.Toolkit.CheckIntegrity(connection);
            m_arrLstBoxIntegrity = new ArrayList();
            m_app                = connection.Application;
			InitializeComponent();
		}

        //******************************************************************
        //
        /// <summary>
        /// Dispose:                    Dispose the form
        /// </summary>
        /// <param name="bDisposing">   true if disposing, false if finalizing</param>
        //
        //******************************************************************
		protected override void Dispose(bool bDisposing){
			if (bDisposing) {
                if (m_arrLstBoxIntegrity.Count != 0) {
                    foreach(ListBox lstBox in m_arrLstBoxIntegrity) {
                        lstBox.Click       -= new EventHandler(lstBoxClick);
                        lstBox.DoubleClick -= new EventHandler(lstBoxDoubleClick);
                    }
                }
				if (components != null) {
					components.Dispose();
				}
			}
			base.Dispose(bDisposing);
		}

        private ListBox LastListBox {
            get {
                ListBox lstBoxRetVal = null;
                int     iCount;

                iCount = m_arrLstBoxIntegrity.Count;
                if (m_arrLstBoxIntegrity.Count != 0) {
                    lstBoxRetVal =  m_arrLstBoxIntegrity[iCount - 1] as ListBox;
                }
                return(lstBoxRetVal);
            }
        }
        //********************************************************************************
        //
        /// <summary>
        /// GetPKColValue :                 Get fields' values list from the columns list
        /// </summary>
        /// <param name="ginst">            Groupinstance to get field values</param>
        /// <param name="arrPKColList">     Columns list</param>
        /// <returns>
        /// Fields' value list
        /// </returns>
        //
        //********************************************************************************
        private string[] GetPKColValue(RepPP.GroupInstance ginst, string[] arrPKColList) {
            RepPP.Field     fld;
            string[]        arrRetVal;

            arrRetVal = new string[arrPKColList.Length];
            for(int i = 0; i < arrPKColList.Length; i++) {
                fld = m_checkIntegrity.FindFldUsingCol(ginst.Group.Fields, arrPKColList[i]);
                if (fld != null) {
                    arrRetVal[i] = fld.Value;
                }
            }
            return(arrRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// MeasureMessage :                Measures the label sizes
        /// </summary>
        //        
        //********************************************************************************
		private Size MeasureMessage(int iMaxWidth) {
			Graphics    graph;
			SizeF       sizeMessage;

            using(graph = CreateGraphics()) {
                sizeMessage = graph.MeasureString(lblIntegMessage.Text, lblIntegMessage.Font, iMaxWidth);			
            }
            return(new Size((int)Math.Ceiling(sizeMessage.Width), (int)Math.Ceiling(sizeMessage.Height)));
		}

        //********************************************************************************
        //
        /// <summary>
        /// SetSizeDialogBox :              Sizes the correct control's  within the Dialog box
        /// </summary>
        //        
        //********************************************************************************
        private void SetSizeDialogBox() {
		    Size        sizeRectMessage;
            int         iDiff;
            int         iMaxWidth;

            iMaxWidth               = pnlTop.ClientSize.Width - SystemInformation.VerticalScrollBarWidth;;
			sizeRectMessage         = MeasureMessage(iMaxWidth);
            sizeRectMessage.Width  += SystemInformation.VerticalScrollBarWidth;
            sizeRectMessage.Height += SystemInformation.HorizontalScrollBarHeight;
            lblIntegMessage.Size    = sizeRectMessage;
            if (sizeRectMessage.Height > pnlTop.Height) {
                iDiff = sizeRectMessage.Height - pnlTop.Height;
                if (Height + iDiff <= Screen.PrimaryScreen.Bounds.Height) {
                    pnlTop.Height += iDiff;                
                    Height        += iDiff;
                }
            }
        }

        private void lstBoxClick(object sender, EventArgs e) {
            OnIntegrityClick(new IntegrityEventArgs(sender as ListBox, m_arrLstBoxIntegrity.IndexOf(sender)));
        }

        private void lstBoxDoubleClick(object sender, EventArgs e) {
            OnIntegrityDoubleClick(new IntegrityEventArgs(sender as ListBox, m_arrLstBoxIntegrity.IndexOf(sender)));
        }

        //********************************************************************************
        //
        /// <summary>
        /// BuildListbox :                  Build IsReferenceBy Listbox
        /// </summary>
        /// <param name="strSubTitle">      SubTitle for listbox (optionnal)</param>
        //
        //********************************************************************************
        private void BuildListbox(string strSubTitle) {
            ListBox     lstBox;  
            Label       label            = null;
            int         iMaxHeightCtrl;
            bool        bCreateSubTitle;

            bCreateSubTitle = (strSubTitle != string.Empty);
            if (LastListBox != null) {
                iMaxHeightCtrl = (bCreateSubTitle) ? HEIGHT_LISTBOX + MARGIN_LISTBOX + HEIGHTLABEL_LISTBOX : 
                                                     HEIGHT_LISTBOX + MARGIN_LISTBOX;
                if (Height + iMaxHeightCtrl <= Screen.PrimaryScreen.Bounds.Height)  {
                    Height += iMaxHeightCtrl;
                }
            } else if (bCreateSubTitle) {
                Height += HEIGHTLABEL_LISTBOX;
            }
            if (bCreateSubTitle) {
                label        = new Label();
                label.Top    = (LastListBox == null) ? 0 : LastListBox.Bottom + MARGIN_LISTBOX;
                label.Height = HEIGHTLABEL_LISTBOX;
                label.Left   = MARGIN_LISTBOX;
                label.Width  = pnlMain.Width - (MARGIN_LISTBOX * 2);;
                label.Font   = new Font(label.Font, System.Drawing.FontStyle.Bold);
                label.Anchor = AnchorStyles.Top | AnchorStyles.Left | AnchorStyles.Right;
                label.Text   = strSubTitle;
                label.Parent = pnlMain;
            }
            lstBox = new ListBox();
            if (label != null) {
                lstBox.Top = (LastListBox == null) ? label.Bottom : label.Bottom;
            } else {
                lstBox.Top = (LastListBox == null) ? 0 : LastListBox.Bottom + MARGIN_LISTBOX;
            }
            lstBox.Height        = HEIGHT_LISTBOX;
            lstBox.Left          = MARGIN_LISTBOX;
            lstBox.SelectionMode = SelectionMode.None;
            lstBox.Width         = pnlMain.Width - (MARGIN_LISTBOX * 2);
            lstBox.Anchor        = AnchorStyles.Top | AnchorStyles.Left | AnchorStyles.Right;
            lstBox.Parent        = pnlMain;
            lstBox.Click        += new EventHandler(lstBoxClick);
            lstBox.DoubleClick  += new EventHandler(lstBoxDoubleClick);
            m_arrLstBoxIntegrity.Add(lstBox);
        }

        //********************************************************************************
        //
        /// <summary>
        /// PadItem :                       Pad the item with the correct max size
        /// </summary>
        /// <param name="graph">            Graphics object</param>
        /// <param name="fMaxSize">         Max size for the item</param>
        /// <param name="strItem">          Item to size</param>
        /// <returns>
        /// Sized item
        /// </returns>
        //        
        //********************************************************************************
        private string PadItem(Graphics graph, string strItem, float fMaxSize) {
            string  strRetVal;
            string  strMeasureItem;
            float   fWidth;
            bool    bContinue = true;

            strRetVal      = strItem;
            strMeasureItem = strRetVal;                
            do {
                fWidth = graph.MeasureString(strMeasureItem, LastListBox.Font).Width;	
                if (fWidth < fMaxSize) {
                    strMeasureItem += "|";
                    strRetVal      += " ";
                } else {
                    bContinue = false;
                }
            } while(bContinue);
            return(strRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// ReformatList :                  Reformat the list to fit in the listbox
        /// </summary>
        /// <param name="arrItems">         Items list to format to the listbox</param>
        /// <returns>
        /// Formated list
        /// </returns>
        //        
        //********************************************************************************
        private string[][] ReformatList(string[][] arrItems) {
			Graphics    graph;
            string[][]  arrRetVal    = null;
            string      strColValue;
            int[]       arrMaxLength = null;
            float[]     arrMaxWidth = null;
            float       fWidth;
            int         i;
           
            graph = CreateGraphics();
            using(graph) {
                foreach (string[] arrColValues in arrItems) {
                    if (arrMaxLength == null) {
                        arrMaxLength = new int[arrColValues.Length];
                        arrMaxWidth  = new float[arrColValues.Length];
                    }
                    for (i = 0; i < arrColValues.Length - 1; i++) {
                        // Optimisation : Pad with max spaces before do job
                        if (arrColValues[i].Length > arrMaxLength[i]) {
                            arrMaxLength[i] = arrColValues[i].Length;
                            fWidth          = graph.MeasureString(arrColValues[i], LastListBox.Font).Width;	
                            if (fWidth > arrMaxWidth[i]) {
                                arrMaxWidth[i] = fWidth;
                            }
                        }
                    }
                }
                arrRetVal = new string[arrItems.Length][];
                for (i = 0; i < arrItems.Length; i++) {                
                    arrRetVal[i] = new string[arrItems[i].Length];
                    for (int j = 0; j < arrItems[i].Length; j++) {
                        strColValue     = arrItems[i][j];                        
                        arrRetVal[i][j] = (j < arrItems[i].Length - 1) ? PadItem(graph, strColValue.PadRight(arrMaxLength[j]), arrMaxWidth[j]) :
                                                                         strColValue;                   
                    }
                }
            }
            return(arrRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// FillIsReferenceByListbox :                  Fill IsReferenceBy Listbox
        /// </summary>
        /// <param name="arrReferencedBy">              IsReferenceBy values</param>
        //
        //********************************************************************************
        private void FillIsReferenceByListbox(string[][] arrReferencedBy) {
            string[][]  arrReformatList;
            string      strColValues;

            LastListBox.Items.Clear();
            arrReformatList = ReformatList(arrReferencedBy);
            foreach(string[] arrColValue in arrReformatList) {
                strColValues = string.Join("\t", arrColValue);
                LastListBox.Items.Add(strColValues);
            } 
        }

        //********************************************************************************
        //
        /// <summary>
        /// CreateMultiDisplayReferencedBy :            Create Multiple 
        ///                                             Display IsReferenceBy
        /// </summary>
        /// <param name="arrReferencedBy">              IsReferenceBy values</param>
        /// <param name="strSubTitle">                  IsReferenceBy Subtitle</param>
        //
        //********************************************************************************
        private void CreateMultiDisplayReferencedBy(string[][]             arrReferencedBy,
                                                    string                 strSubTitle) {
            BuildListbox(strSubTitle);            
            FillIsReferenceByListbox(arrReferencedBy);                
        }

        //********************************************************************************
        //
        /// <summary>
        /// CreateDisplayReferencedBy :                 Create Single 
        ///                                             Display IsReferenceBy
        /// </summary>
        /// <param name="arrReferencedBy">              IsReferenceBy values</param>
        //
        //********************************************************************************
        private void CreateDisplayReferencedBy(string[][] arrReferencedBy) {
            BuildListbox("");
            FillIsReferenceByListbox(arrReferencedBy);
        }

        //********************************************************************************
        //
        /// <summary>
        /// IsRefencedBy :                  Check if item in Groupinstance is referenced by one or more foreign tables
        /// </summary>
        /// <param name="ginst">            Groupinstance to check</param>
        /// <param name="strTableName">     Table name containing the item to check</param>
        /// <param name="arrPKColList">     Reference Columns where to check</param>
        /// <param name="arrPKFldList">     Field containing the value to check. If null, the field attached to arrPKColList are used.</param>
        /// <param name="arrRetColList">    List of columns to select</param>
        /// <param name="strMainTitle">     Main Title for the dialog box</param>                            
        /// <param name="strMainMessage">   Main Message to display if one or more integrity 
        ///                                 has been found</param>
        /// <returns>
        /// true if one or more integrities has been found
        /// </returns>
        //  SELECT arrRetColList FROM strTableName WHERE arrPKColList = :GINST.arrPKFldList
        //  SELECT QUNAME FROM SYSTQUEST WHERE APPSYSTNAME=:GINST.APPSYSTNAME AND COLNAME=:GINST.COLNAME
        //
        //********************************************************************************
        virtual public bool IsRefencedBy(RepPP.GroupInstance    ginst, 
                                         string                 strTableName, 
                                         string[]               arrPKColList, 
                                         RepPP.Field[]          arrPKFldList,
                                         string[]               arrRetColList,
                                         string                 strMainTitle,
                                         string                 strMainMessage) {
            string[][]  arrReferencedBy;
            bool        bRetVal;

            bRetVal = m_checkIntegrity.IsReferencedBy(ginst, strTableName, arrPKColList, arrPKFldList, arrRetColList, out arrReferencedBy);
            if (bRetVal) {
                this.Text            = strMainTitle;
                lblIntegMessage.Text = string.Format(strMainMessage, GetPKColValue(ginst, arrPKColList));
                CreateDisplayReferencedBy(arrReferencedBy);
                SetSizeDialogBox();
                ShowDialog();
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// IsRefencedBy :                  Check if item in Groupinstance is referenced by one or more foreign tables
        /// </summary>
        /// <param name="ginst">            Groupinstance to check</param>
        /// <param name="strTableName">     Table name containing the item to check</param>
        /// <param name="arrPKColList">     Reference Columns where to check</param>
        /// <param name="arrRetColList">    List of columns to select</param>
        /// <param name="strMainTitle">     Main Title for the dialog box</param>                            
        /// <param name="strMainMessage">   Main Message to display if one or more integrity 
        ///                                 has been found</param>
        /// <returns>
        /// true if one or more integrities has been found
        /// </returns>
        //  SELECT arrRetColList FROM strTableName WHERE arrPKColList = :GINST.arrPKFldList
        //  SELECT QUNAME FROM SYSTQUEST WHERE APPSYSTNAME=:GINST.APPSYSTNAME AND COLNAME=:GINST.COLNAME
        //
        //********************************************************************************
        virtual public bool IsRefencedBy(RepPP.GroupInstance    ginst, 
                                         string                 strTableName, 
                                         string[]               arrPKColList, 
                                         string[]               arrRetColList,
                                         string                 strMainTitle,
                                         string                 strMainMessage) {
            return(IsRefencedBy(ginst, strTableName, arrPKColList, null, arrRetColList, strMainTitle, strMainMessage));
        }

        //********************************************************************************
        //
        /// <summary>
        /// IsRefencedBy :                  Check if item in Groupinstance is referenced by one or more foreign tables
        /// </summary>
        /// <param name="ginst">            Groupinstance to check</param>
        /// <param name="arrTableName">     Array of Table name containing the item to check</param>
        /// <param name="arrPKColList">     Array of array of Reference Columns where to check</param>
        /// <param name="arrPKFldList">     Array of array of Field containing the value to check. If null, the field attached to arrPKColList are used.</param>
        /// <param name="arrRetColList">    Array of array of columns to select</param>
        /// <param name="strMainTitle">     Main Title for the dialog box</param>   
        /// <param name="arrSubTitle">      Array of subtitle for IsReferenceBy Listbox</param>                         
        /// <param name="strMainMessage">   Main Message to display if one or more 
        ///                                 integrity has been found</param>
        /// <returns>
        /// true if one or more integrities has been found
        /// </returns>
        //  SELECT arrRetColList FROM strTableName WHERE arrPKColList = :GINST.arrPKFldList
        //  SELECT QUNAME FROM SYSTQUEST WHERE APPSYSTNAME=:GINST.APPSYSTNAME AND COLNAME=:GINST.COLNAME
        //
        //********************************************************************************
        virtual public bool IsRefencedBy(RepPP.GroupInstance    ginst, 
                                         string[]               arrTableName, 
                                         string[][]             arrPKColList, 
                                         RepPP.Field[][]        arrPKFldList,    
                                         string[][]             arrRetColList,
                                         string                 strMainTitle,
                                         string[]               arrSubTitle,
                                         string                 strMainMessage) {
            string[][]      arrReferencedBy;
            RepPP.Field[]   arrPKFld;
            bool            bRetVal         = false;

            if (arrTableName.Length != arrPKColList.Length) {
                throw new Exception("IsReferencedBy : The table name list doesn't fit with the columns primary keys list");
            }
            for(int i = 0; i < arrTableName.Length; i++) {
                arrPKFld = (arrPKFldList != null) ? arrPKFldList[i] : null;
                if (m_checkIntegrity.IsReferencedBy(ginst, arrTableName[i], arrPKColList[i], arrPKFld, arrRetColList[i], out arrReferencedBy)) {
                    CreateMultiDisplayReferencedBy(arrReferencedBy, arrSubTitle[i]);
                    bRetVal = true;
                }
            }
            if (bRetVal) {
                this.Text            = strMainTitle;
                lblIntegMessage.Text = string.Format(strMainMessage, GetPKColValue(ginst, arrPKColList[0]));
                SetSizeDialogBox();
                ShowDialog();
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// IsRefencedBy :                  Check if item in Groupinstance is referenced by one or more foreign tables
        /// </summary>
        /// <param name="ginst">            Groupinstance to check</param>
        /// <param name="arrTableName">     Array of Table name containing the item to check</param>
        /// <param name="arrPKColList">     Array of array of Reference Columns where to check</param>
        /// <param name="arrRetColList">    Array of array of columns to select</param>
        /// <param name="strMainTitle">     Main Title for the dialog box</param>   
        /// <param name="arrSubTitle">      Array of subtitle for IsReferenceBy Listbox</param>                         
        /// <param name="strMainMessage">   Main Message to display if one or more 
        ///                                 integrity has been found</param>
        /// <returns>
        /// true if one or more integrities has been found
        /// </returns>
        //  SELECT arrRetColList FROM strTableName WHERE arrPKColList = :GINST.arrPKFldList
        //  SELECT QUNAME FROM SYSTQUEST WHERE APPSYSTNAME=:GINST.APPSYSTNAME AND COLNAME=:GINST.COLNAME
        //
        //********************************************************************************
        public bool IsRefencedBy(RepPP.GroupInstance    ginst, 
                                 string[]               arrTableName, 
                                 string[][]             arrPKColList, 
                                 string[][]             arrRetColList,
                                 string                 strMainTitle,
                                 string[]               arrSubTitle,
                                 string                 strMainMessage) {
            return(IsRefencedBy(ginst, arrTableName, arrPKColList, null, arrRetColList, strMainTitle, arrSubTitle, strMainMessage));
        }

        //********************************************************************************
        //
        /// <summary>
        /// IsItemExist :                   Check if item in Groupinstance is 
        ///                                 exist (Referencing)
        /// </summary>
        /// <param name="ginst">            Groupinstance to check</param>
        /// <param name="strFKTableName">   Foreign Table name where to check</param>
        /// <param name="arrFKTablePK">     Primary key of the foreign table.</param>
        /// <param name="arrFldToCheck">    Field where to check in case. If null, the primary key is used</param>
        /// <param name="strMainTitle">     Main Title for the dialog box</param>                            
        /// <param name="strMainMessage">   Main Message to display if one or more integrity 
        ///                                 has been found</param>
        /// <returns>
        /// true if one or more reference has been found
        /// </returns>
        /// SELECT COUNT(*) FROM strFKTableName WHERE arrFKTablePK=:Field attached to arrFKTablePK
        //
        //********************************************************************************
        public bool IsItemExist(RepPP.GroupInstance    ginst, 
                                string                 strFKTableName, 
                                string[]               arrFKTablePK, 
                                RepPP.Field[]          arrFldToCheck,
                                string                 strMainTitle,
                                string                 strMainMessage) {
            bool        bRetVal;

            bRetVal = m_checkIntegrity.IsItemExist(ginst, strFKTableName, arrFKTablePK, arrFldToCheck);
            if (bRetVal) {
                MessageBox.Show(string.Format(strMainMessage, GetPKColValue(ginst, arrFKTablePK)), strMainTitle);
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// IsItemExist :                   Check if item in Groupinstance is 
        ///                                 exist (Referencing)
        /// </summary>
        /// <param name="ginst">            Groupinstance to check</param>
        /// <param name="strFKTableName">   Foreign Table name where to check</param>
        /// <param name="arrFKTablePK">     Primary key of the foreign table.</param>
        /// <param name="strMainTitle">     Main Title for the dialog box</param>                            
        /// <param name="strMainMessage">   Main Message to display if one or more integrity 
        ///                                 has been found</param>
        /// <returns>
        /// true if one or more reference has been found
        /// </returns>
        /// SELECT COUNT(*) FROM strFKTableName WHERE arrFKTablePK=:Field attached to arrFKTablePK
        //
        //********************************************************************************
        public bool IsItemExist(RepPP.GroupInstance    ginst, 
                                string                 strFKTableName, 
                                string[]               arrFKTablePK, 
                                string                 strMainTitle,
                                string                 strMainMessage) {
            return(IsItemExist(ginst, strFKTableName, arrFKTablePK, null, strMainTitle, strMainMessage));
        }

        //********************************************************************************
        //
        /// <summary>
        /// IsPKExist :                     Check if the PK exist
        /// </summary>
        /// <param name="ginst">            Groupinstance to check</param>
        /// <param name="strMainTitle">     Main Title for the dialog box</param>                            
        /// <param name="strMainMessage">   Main Message to display if one or more integrity 
        ///                                 has been found</param>
        /// <returns>
        /// true if one or more reference has been found
        /// </returns>
        //
        //********************************************************************************
        public bool IsPKExist(RepPP.GroupInstance    ginst,
                              string                 strMainTitle,
                              string                 strMainMessage) {
            bool        bRetVal;

            bRetVal = m_checkIntegrity.IsPKExist(ginst);
            if (bRetVal) {
                MessageBox.Show(strMainMessage, strMainTitle);
            }
            return(bRetVal);
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnIntegrityClick:           Called when the integrity listbox is clicked
        /// </summary>
        //
        //********************************************************************************
        protected virtual void OnIntegrityClick(IntegrityEventArgs e) {
            if (IntegrityClick != null) {
                IntegrityClick(this, e);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnIntegrityDoubleClick:     Called when the integrity listbox is double-clicked
        /// </summary>
        //
        //********************************************************************************
        protected virtual void OnIntegrityDoubleClick(IntegrityEventArgs e) {
            if (IntegrityDoubleClick != null) {
                IntegrityDoubleClick(this, e);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// IntegrityEventArgs :        Event Arguments for the clicked Integrities 
        /// </summary>
        //
        //********************************************************************************
        public class IntegrityEventArgs : System.EventArgs {
            /// <summary>Listbox integrity clicked</summary>
            private ListBox                 m_lstBox;
            /// <summary>The index of the integrity clicked (Multi-integrity)</summary>
            private int                     m_iIndex;

            //*********************************************************
            //
            /// <summary>
            /// ctor:                       Class constructor
            /// </summary>
            //
            //*********************************************************
            public IntegrityEventArgs(ListBox lstBox, int iIndex) {
                m_lstBox = lstBox;
                m_iIndex = iIndex;
            }

            //********************************************************************************
            //
            /// <summary>
            /// ListBoxIntegrity :              Return the Listbox integrity clicked
            /// </summary>
            //
            //********************************************************************************
            public ListBox ListBoxIntegrity {
                get {
                    return(m_lstBox);
                }
            }

            //********************************************************************************
            //
            /// <summary>
            /// IndexIntegrity :                 Return the index of the  integrity clicked
            ///                                  (Multi-integrity)
            /// </summary>            /// 
            //
            //********************************************************************************
            public int IndexIntegrity {
                get {
                    return(m_iIndex);
                }
            }        
        } // Class IntegrityEventArgs 

        #region Windows Form Designer generated code

		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		private void InitializeComponent()
		{
            this.pnlTop = new System.Windows.Forms.Panel();
            this.lblIntegMessage = new System.Windows.Forms.Label();
            this.pnlMain = new System.Windows.Forms.Panel();
            this.pnlBottom = new System.Windows.Forms.Panel();
            this.btnOK = new System.Windows.Forms.Button();
            this.pnlTop.SuspendLayout();
            this.pnlBottom.SuspendLayout();
            this.SuspendLayout();
            // 
            // pnlTop
            // 
            this.pnlTop.Controls.Add(this.lblIntegMessage);
            this.pnlTop.Dock = System.Windows.Forms.DockStyle.Top;
            this.pnlTop.Location = new System.Drawing.Point(0, 0);
            this.pnlTop.Name = "pnlTop";
            this.pnlTop.Size = new System.Drawing.Size(418, 32);
            this.pnlTop.TabIndex = 6;
            // 
            // lblIntegMessage
            // 
            this.lblIntegMessage.Location = new System.Drawing.Point(8, 8);
            this.lblIntegMessage.Name = "lblIntegMessage";
            this.lblIntegMessage.Size = new System.Drawing.Size(312, 16);
            this.lblIntegMessage.TabIndex = 6;
            this.lblIntegMessage.Text = "Integrity Message";
            // 
            // pnlMain
            // 
            this.pnlMain.AutoScroll = true;
            this.pnlMain.Dock = System.Windows.Forms.DockStyle.Fill;
            this.pnlMain.Location = new System.Drawing.Point(0, 32);
            this.pnlMain.Name = "pnlMain";
            this.pnlMain.Size = new System.Drawing.Size(418, 160);
            this.pnlMain.TabIndex = 7;
            // 
            // pnlBottom
            // 
            this.pnlBottom.Controls.Add(this.btnOK);
            this.pnlBottom.Dock = System.Windows.Forms.DockStyle.Bottom;
            this.pnlBottom.Location = new System.Drawing.Point(0, 192);
            this.pnlBottom.Name = "pnlBottom";
            this.pnlBottom.Size = new System.Drawing.Size(418, 48);
            this.pnlBottom.TabIndex = 8;
            // 
            // btnOK
            // 
            this.btnOK.DialogResult = System.Windows.Forms.DialogResult.OK;
            this.btnOK.Location = new System.Drawing.Point(169, 12);
            this.btnOK.Name = "btnOK";
            this.btnOK.Size = new System.Drawing.Size(80, 24);
            this.btnOK.TabIndex = 2;
            this.btnOK.Text = "OK";
            // 
            // CheckIntegrityUI
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(418, 240);
            this.Controls.Add(this.pnlMain);
            this.Controls.Add(this.pnlBottom);
            this.Controls.Add(this.pnlTop);
            this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedToolWindow;
            this.MaximizeBox = false;
            this.MinimizeBox = false;
            this.Name = "CheckIntegrityUI";
            this.ShowInTaskbar = false;
            this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
            this.Text = "Integrity Check";
            this.pnlTop.ResumeLayout(false);
            this.pnlBottom.ResumeLayout(false);
            this.ResumeLayout(false);

        }
		#endregion

    } // Class CheckIntegrityUI
} // Namespace
