using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Xml;
using sdfwwebcsnet;
using System.Web;
using System.IO;
using RepPP.Toolkit;

namespace RepPP.VSNetWizard.ASPNet
{
	/// <summary>
	/// GeneratorForVB:     Generator for VB
	/// </summary>
	internal class GeneratorForVB : GeneratorBase {

        //******************************************************************
        //
        /// <summary>
        /// ctor:                               Class constructor
        /// </summary>
        /// <param name="vsApp">                Visual studio application</param>
        /// <param name="app">                  Rep++ application object</param>
        /// <param name="genTypedInstParam">    Generate typed instances parameters</param>
        /// <param name="strTemplFileName">     HTML template file name</param>
        /// <param name="eReplaceMode">         Replacing file mode</param>
        //
        //******************************************************************
		public GeneratorForVB(EnvDTE.DTE                vsApp,
                              RepPP.IApplication        app,
                              Utility.GenTypedInstParam  genTypedInstParam,
                              string                    strTemplFileName,
                              Utility.ReplaceFileModeE  eReplaceMode) : base (vsApp,
                                                                               app,
                                                                               genTypedInstParam,
                                                                               strTemplFileName,
                                                                               eReplaceMode)
		{
        }

        //*********************************************************     
        //
        /// <summary>
        /// CreateProject:                  Create a new visual studio project
        /// </summary>
        /// <param name="strPrjName">       Project name</param>
        /// <param name="strPrjFullName">   Project full name</param>
        /// <returns>
        /// The created project
        /// </returns>
        //  
        //*********************************************************     
        protected override EnvDTE.Project CreateProject(ref string strPrjName, ref string strPrjFullName) {
            EnvDTE.Project  project;

            project = Utility.CreateWebProject(m_vsApp,
                                               ProgramLanguageE.sdVisualBasic,
                                               ref strPrjName,
                                               ref strPrjFullName);
            Utility.DeleteProjectItem(project, "Styles.css");
            Utility.DeleteProjectItem(project, "Global.asax");
            return(project);
        }

        //*********************************************************     
        //
        /// <summary>
        /// CodeBehindExtension:        Get the code behin extension
        /// </summary>
        //
        //*********************************************************     
        protected override string CodeBehindExtension {
            get {
                return(".vb");
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// GenCardTypedInstance:           Genearte card typed instance 
        /// </summary>
        /// <param name="strPrjName">       Project name</param>
        /// <param name="strCardName">      Card name</param>
        //
        //*********************************************************     
        protected override void GenCardTypedInstance(string strPrjName, string strCardName) {
            string  strTISubFolder;

#if VC8
            strTISubFolder = "App_Code";
#else
            strTISubFolder = String.Empty;
#endif
            Utility.GenCardTypedInstance(Utility.GetProject(m_vsApp, strPrjName),
                                         m_genTypedInstParam.m_strCardInstClassPrefix,
                                         m_genTypedInstParam.m_strGroupInstClassPrefix,
                                         m_genTypedInstParam.m_strGroupInstPropPrefix,
                                         m_genTypedInstParam.m_strFieldPropPrefix,
                                         m_genTypedInstParam.m_strNamespace,
                                         m_genTypedInstParam.m_strRootNamespace,
                                         m_genTypedInstParam.m_strCardInstBaseClass,
                                         m_genTypedInstParam.m_strGroupInstBaseClass,
                                         new TypedInstanceGenForVB(),
                                         (m_genTypedInstParam.m_eGenType == Utility.GenTypedInstTypeE.TypedInstAndWSAdapter),
                                         m_app.Cards[strCardName],
                                         m_eReplaceFileMode,
                                         strTISubFolder,
                                         true);
        }

        //*********************************************************     
        //
        /// <summary>
        /// GenGroupTypedInstance:          Generate group typed instance 
        /// </summary>
        /// <param name="strPrjName">       Project name</param>
        /// <param name="strCardName">      Group name</param>
        //
        //*********************************************************     
        protected override void GenGroupTypedInstance(string strPrjName, string strGroupName) {
            string  strTISubFolder;

#if VC8
            strTISubFolder = "App_Code";
#else
            strTISubFolder = String.Empty;
#endif
            Utility.GenGroupTypedInstance(Utility.GetProject(m_vsApp, strPrjName),
                                          m_genTypedInstParam.m_strCardInstClassPrefix,
                                          m_genTypedInstParam.m_strGroupInstClassPrefix,
                                          m_genTypedInstParam.m_strGroupInstPropPrefix,
                                          m_genTypedInstParam.m_strFieldPropPrefix,
                                          m_genTypedInstParam.m_strNamespace,
                                          m_genTypedInstParam.m_strRootNamespace,
                                          m_genTypedInstParam.m_strGroupInstBaseClass,
                                          new TypedInstanceGenForVB(),
                                          (m_genTypedInstParam.m_eGenType == Utility.GenTypedInstTypeE.TypedInstAndWSAdapter),
                                          m_app.Groups[strGroupName],
                                          m_eReplaceFileMode,
                                          strTISubFolder);
        }
	} // class GeneratorForVB
} // namespace
