using System;
using EnvDTE;
using System.IO;
using VSLangProj;
using Microsoft.Win32;
using RepPP.Toolkit;
using RepPP.VSNetWizard;
using System.CodeDom;

//**********************************************************************
//
//    Module:             WinGenerateforVB.cs.
//
//    Author:             Maher Najah
//    Version:            1.00.00
//    Creation:           2004-11-25
//    Modification:       2004-11-25
//
//    Description:        Implementing VB Generation code
//
//**********************************************************************
namespace RepPP.VSNetWizard.Windows {
    /// <summary>
    /// Summary description for WinGenerateforVB.
    /// </summary>
    internal class WinGeneratorForVB : WinGeneratorBase {
        //**********************************************************************
        //
        // Constructor
        //   In: 
        //       vsApp      :    Visual studio application</param>
        //       app        :    Rep++ application object</param>
        //       oVSInst    :    DTE
        //       oRep       :    The Rep++ Application
        //       lblStatus  :    Status label  
        //       bUseBar    :    True to use .Net Bar components
        //       bGenWrapper:    True to generate group instance wrapper
        //   Returns:
        //
        //**********************************************************************
        public WinGeneratorForVB(EnvDTE.DTE                 vsApp,
                                 RepPP.IApplication         oRep, 
                                 System.Windows.Forms.Label lblStatus, 
                                 bool                       bUseBar, 
                                 Utility.GenTypedInstParam  genTypedInstParam) : base(vsApp, 
                                                                                      oRep,
                                                                                      lblStatus,
                                                                                      bUseBar,
                                                                                      genTypedInstParam) {
        }

        //**********************************************************************
        //
        // Create new Project and set references
        //   In: 
        //       strProjectDir :    The project directory
        //       strProjectName:    The project name
        //   Returns:
        //       The created project
        //
        //**********************************************************************
        protected override Project CreateProject(string  strProjectDir, string  strProjectName) {
            EnvDTE.Project project;

#if VC8
            project = Utility.GetProject(m_vsApp, strProjectName);
#else            
            project = Utility.CreateWindowsProject(m_vsApp,
                                                   ProgramLanguageE.sdVisualBasic,
                                                   ref strProjectName,
                                                   ref strProjectDir);
            foreach (EnvDTE.Property aProp in project.Properties) {
                if (aProp.Name == "StartupObject") {
                    aProp.Value = "Sub Main";
                    break;
                }
            }
#endif
            return(project);
        }

        //**********************************************************************
        //
        // Add a control to the form
        //   In: 
        //       strProjectName :    The project in witch the control have to be added
        //       strFormName    :    The form in witch the control have to be added
        //       strCtlName     :    The control name
        //       strCtlType     :    The control type
        //       aPropNames     :    Array of property names
        //       aPropTypes     :    Array of property types
        //       aPropValues    :    Array of property values
        //       strParentName  :    The parent in witch the control have to be added
        //       strCollName    :    The collection name in witch the control have to 
        //                           to be added (default value: Controls)
        //       strCollType    :    The collection Type (default value: Control)
        //       bDeclareIt     :    Control declaration.
        //   Returns:
        //
        //**********************************************************************
        public override void AddControl(string      strProjectName, 
                                        string      strFormName, 
                                        string      strCtlName, 
                                        string      strCtlType, 
                                        object[]    aPropNames, 
                                        object[]    aPropTypes, 
                                        object[]    aPropValues, 
                                        string      strParentName, 
                                        string      strCollName, 
                                        string      strCollType, 
                                        bool        bDeclareIt) {
            Project         oProject;
            EditPoint       epStart;
            EditPoint       epEnd;
            ProjectItem     projItem;
            CodeClass       codeForm;
            CodeFunction    codeFunc;
            string          strTmp;
            int             iIndex;
            string          strTmp2;

            oProject = GetProject(strProjectName);
            projItem = oProject.ProjectItems.Item(strFormName + ".vb");
#if VC8
            projItem = projItem.ProjectItems.Item(strFormName + ".Designer.vb");
#endif
            codeForm = projItem.FileCodeModel.CodeElements.Item(1) as CodeClass;
            codeFunc = (CodeFunction)codeForm.Members.Item("InitializeComponent");
            epStart  = codeFunc.StartPoint.CreateEditPoint();
            epEnd    = codeFunc.EndPoint.CreateEditPoint();
            // Add control declaration
            epStart.StartOfLine();
            if (bDeclareIt) {
                epStart.Insert("\t" + "Friend WithEvents " + strCtlName + " As " + strCtlType + "\r\n");
            }
            // Add Control initialization
            FindText(ref epStart, epEnd, "'");
            epStart.StartOfLine();
            if (bDeclareIt) {
                if (strCtlType == "RepPP.RepPPInfo") {
                    epStart.Insert("\t" + "\t" + "\tMe." + strCtlName + " = New " + strCtlType + "(components)" + "\r\n");
                } else {
		            epStart.Insert("\t" + "\t" + "\tMe." + strCtlName + " = new " + strCtlType + "()" + "\r\n");
                }
            }
            strTmp = "\t" + "\t" + "'" + "\r\n" + "\t" + "\t" + "'" + strCtlName + "\r\n" + "\t" + "\t" + "'" + "\r\n";
            epStart.Insert(strTmp);
            if (aPropNames != null) {
                for (iIndex = 0; iIndex <= aPropNames.Length - 1; iIndex++) {
                    epStart.Insert("\t" + "\t" + FormatPropLine(strCtlName, aPropNames[iIndex].ToString(), aPropTypes[iIndex].ToString(), aPropValues[iIndex].ToString()));
                    epStart.Insert("\r\n");
                }
            }
            // Add TabIndex property
	        if (strCtlType != "MainMenu"        && 
                strCtlType != "MenuItem"        && 
                strCtlType != "StatusBar"       && 
                strCtlType != "StatusBarPanel"  &&
                strCtlType != "RepPP.RepPPInfo" ) {
                epStart.Insert("\t" + "\t" + FormatPropLine(strCtlName, "TabIndex", "Integer", m_iTabIndex.ToString()));
                epStart.Insert("\r\n");
                m_iTabIndex = m_iTabIndex + 1;
            }
            if (strCtlType == "MainMenu") {
                strTmp = "'" + strFormName;
                if (FindText(ref epStart, epEnd, strTmp)) {
                    epStart.LineDown(2);
                    epStart.Insert("Me.Menu = " + strCtlName + "\r\n" + "\t" + "\t");
                }
            } else if (strCtlType != "RepPP.RepPPInfo") { 
                // Add the control to his parent
                if (strParentName == null || strParentName == "" ) {
                    strTmp = "Me." + strCollName + ".AddRange(New System.Windows.Forms." + strCollType + "() {";
                    strTmp2 = "'" + strFormName;
                } else {
                    strTmp = "Me." + strParentName + "." + strCollName + ".AddRange(New System.Windows.Forms." + strCollType + "() {";
                    strTmp2 = "'" + strParentName;
                }
                codeFunc = (CodeFunction)codeForm.Members.Item("InitializeComponent");
                if (ReplaceFuncText(codeFunc, strTmp, strTmp + strCtlName + ",")) {
                    //do nothing
                } else if (FindText(ref epStart, epEnd, strTmp2)) {
                    epStart.LineDown(2);
                    epStart.Insert(strTmp + "Me." + strCtlName + "})" + "\r\n" + "\t" + "\t");
                }
            }
        }

        //**********************************************************************
        //
        // Add a control to the form
        //   In: 
        //       strProjectName :    The project in witch the control have to be added
        //       strFormName    :    The form in witch the control have to be added
        //       strCtlName     :    The control name
        //       strCtlType     :    The control type
        //       aPropNames     :    Array of property names
        //       aPropTypes     :    Array of property types
        //       aPropValues    :    Array of property values
        //   Returns:
        //
        //**********************************************************************
        public override void AddControl(string   strProjectName, 
                                        string   strFormName, 
                                        string   strCtlName, 
                                        string   strCtlType, 
                                        object[] aPropNames, 
                                        object[] aPropTypes, 
                                        object[] aPropValues) {
            AddControl(strProjectName, 
                       strFormName, 
                       strCtlName, 
                       strCtlType, 
                       aPropNames, 
                       aPropTypes, 
                       aPropValues, 
                       null, 
                       "Controls", 
                       "Control", 
                       true);
        }

        //**********************************************************************
        //
        // Add a control to the form
        //   In: 
        //       strProjectName :    The project in witch the control have to be added
        //       strFormName    :    The form in witch the control have to be added
        //       strCtlName     :    The control name
        //       strCtlType     :    The control type
        //       aPropNames     :    Array of property names
        //       aPropTypes     :    Array of property types
        //       aPropValues    :    Array of property values
        //       strParentName  :    The parent in witch the control have to be added
        //       strCollName    :    The collection name in witch the control have to 
        //                           to be added (default value: Controls)
        //   Returns:
        //
        //**********************************************************************
        public override void AddControl(string   strProjectName, 
                                        string   strFormName, 
                                        string   strCtlName, 
                                        string   strCtlType, 
                                        object[] aPropNames, 
                                        object[] aPropTypes, 
                                        object[] aPropValues, 
                                        string   strParentName, 
                                        string   strCollName) {
            AddControl(strProjectName, 
                       strFormName, 
                       strCtlName, 
                       strCtlType, 
                       aPropNames, 
                       aPropTypes, 
                       aPropValues, 
                       strParentName, 
                       strCollName, 
                       "Control", 
                       true);
        }

        //**********************************************************************
        //
        // Add a control to the form
        //   In: 
        //       strProjectName :    The project in witch the control have to be added
        //       strFormName    :    The form in witch the control have to be added
        //       strCtlName     :    The control name
        //       strCtlType     :    The control type
        //       aPropNames     :    Array of property names
        //       aPropTypes     :    Array of property types
        //       aPropValues    :    Array of property values
        //       strParentName  :    The parent in witch the control have to be added
        //       strCollName    :    The collection name in witch the control have to 
        //                           to be added (default value: Controls)
        //       strCollType    :    The collection Type (default value: Control)
        //   Returns:
        //
        //**********************************************************************
        public override void AddControl(string   strProjectName, 
                                        string   strFormName, 
                                        string   strCtlName, 
                                        string   strCtlType, 
                                        object[] aPropNames, 
                                        object[] aPropTypes, 
                                        object[] aPropValues, 
                                        string   strParentName, 
                                        string   strCollName, 
                                        string   strCollType) {
            AddControl(strProjectName, 
                       strFormName, 
                       strCtlName, 
                       strCtlType, 
                       aPropNames, 
                       aPropTypes, 
                       aPropValues, 
                       strParentName, 
                       strCollName, 
                       strCollType, 
                       true);
        }

        //**********************************************************************
        //
        // Add a form to the project
        //   In: 
        //       strProjectName :    The project name
        //       strFormName    :    The form name
        //       strBaseformName:    The base form name
        //       aPropNames     :    Array of property names
        //       aPropTypes     :    Array of property types
        //       aPropValues    :    Array of property values
        //   Returns:
        //
        //**********************************************************************
        public override void AddForm(string   strProjectName, 
                                     string   strFormName, 
                                     string   strBaseFormName, 
                                     object[] aPropNames, 
                                     object[] aPropTypes, 
                                     object[] aPropValues) {
            string          strTemplate;
            Project         oProject;
            EditPoint       ep;
            ProjectItem     projItem;
            CodeClass       codeForm;
            CodeFunction    codeFunc;
            string          strTmp;
            int             iIndex;
            string          strFileName;

            base.AddForm(strProjectName, 
                         strFormName + ".vb",
                         strBaseFormName,
                         aPropNames,
                         aPropTypes,
                         aPropValues);
            m_iTabIndex = 0;
            oProject    = GetProject(strProjectName);
            // GPC4218:
            // Using visual studio wizard to create a window form can fail
            // in some cases. We create a new form from the visual studio window
            // form template file. 

            strTemplate = Utility.GetFormTemplateFileName(false, ProgramLanguageE.sdVisualBasic);
	        oProject.ProjectItems.AddFromTemplate(strTemplate, strFormName + ".vb");
            strFileName = Path.Combine(Path.GetDirectoryName(oProject.FullName), strFormName + ".vb");
            ReplaceFormTags(strFileName, strFormName, strProjectName);
#if VC8
            strTemplate = Utility.GetFormTemplateFileName(true, ProgramLanguageE.sdVisualBasic);
            oProject.ProjectItems.AddFromTemplate(strTemplate, strFormName + ".Designer.vb");
            strFileName = Path.Combine(Path.GetDirectoryName(oProject.FullName), strFormName + ".Designer.vb");
            ReplaceFormTags(strFileName, strFormName, strProjectName);
#endif
            // change the base class name
            projItem = oProject.ProjectItems.Item(strFormName + ".vb");
#if VC8
            projItem = projItem.ProjectItems.Item(strFormName + ".Designer.vb");
#endif
            codeForm = projItem.FileCodeModel.CodeElements.Item(1) as CodeClass;
            // Not implemented
            // codeForm.RemoveBase("System.Windows.Forms.Form")
            // codeForm.AddBase(strBaseFormName)
            ReplaceText(codeForm.StartPoint.CreateEditPoint(), 
                        codeForm.EndPoint.CreateEditPoint(), 
                        "Inherits System.Windows.Forms.Form", 
                        "Inherits " + strBaseFormName);
            // Add form initialization
            codeFunc = (CodeFunction)codeForm.Members.Item("InitializeComponent");                   
            ep = codeFunc.EndPoint.CreateEditPoint();
            ep.StartOfLine();
            ep.LineUp(1);
            strTmp = "\t" + "\t" + "'" + "\r\n" + "\t" + "\t" + "'" + strFormName + "\r\n" + "\t" + "\t" + "'" + "\r" + "\n";
            ep.Insert(strTmp);
            for (iIndex=0; iIndex <= aPropNames.Length - 1;iIndex++) {
                ep.Insert("\t" + "\t" + FormatPropLine("Me", aPropNames[iIndex].ToString(), aPropTypes[iIndex].ToString(), aPropValues[iIndex].ToString()));
			    ep.Insert("\r" + "\n");
            }
        }

        //**********************************************************************
        //
        // AddMainFunction:             Add the main function
        //   In: 
        //       strProjectName   :     The project name
        //       strMainFormName  :     Main form name
        //   Returns:
        //
        //**********************************************************************
        private void AddMainFunction(string strProjectName, string strMainFormName) {
            
#if VC8
            ReplaceLineInProjItem(strProjectName, 
                                  "Application.Designer.vb",
                                  "Me.MainForm",
                                  "Me.MainForm = Global." + strProjectName + "." + strMainFormName + "\r\n");
            ReplaceLineInProjItem(strProjectName, 
                                  "Application.myapp",
                                  "<MainForm>",
                                  "<MainForm>" + strMainFormName + "</MainForm>\r\n");
#else
            string  strText;

            strText = "\t" + "\t" + "System.Windows.Forms.Application.EnableVisualStyles()" + "\r\n" + 
                      "\t" + "\t" + "System.Windows.Forms.Application.DoEvents()" + "\r\n" + 
                      "\t" + "\t" + "System.Windows.Forms.Application.Run(New " + strMainFormName + "())";
	        AddFunction(strProjectName, 
                        strMainFormName, 
                        "Main",
                        vsCMFunction.vsCMFunctionShared,
                        "", 
                        new object[] {"args()"}, 
                        new object[] {"String"},
                        "    Main function",
                        "",
                        null,
                        vsCMAccess.vsCMAccessPublic,
                        strText);
#endif
        }

        //**********************************************************************
        //
        // Add a selection buffer form to the project
        //   In: 
        //       strProjectName   :    The project name
        //       strCardName      :    The Card name of the child editor
        //       strSBFormName    :    The selection buffer form name
        //       strSBCardName    :    The selection buffer card name
        //       strChildFormName :    The child form name
        //       strClassWrapName:     The wrapper class name for the card instance
        //   Returns:
        //
        //**********************************************************************
        protected override void AddMainSBForm(string strProjectName, 
                                              string strCardName, 
                                              string strSBFormName, 
                                              string strSBCardName, 
                                              string strChildFormName, 
                                              string strClassWrapName)
        {

	        string strBody;

	        // Add Typed instance for SB card
	        if (m_genTypedInstParam.m_eGenType != Utility.GenTypedInstTypeE.None) {
                //TODO: Add a typed instance menber in the class for the selection buffer card
		        GenCardTypedInstance(strProjectName, strSBCardName);
	        }
	        // Add the main form
	        AddForm(strProjectName, 
                    strSBFormName, 
                    ((m_bUseDotNetBar) ? "RepPP.Framework.Window.Ext.frmListViewSBEditor" : "RepPP.Framework.Window.frmSBEditorListView"), 
                    new object[] {"Name", "ClientSize"}, 
                    new object[] {"String", "Object"}, 
                    new object[] {strSBFormName, "Size(700, 580)"});
		    // Add Rep++ info component to the form
            AddControl(strProjectName, 
                       strSBFormName, 
                       "repInfo", 
                       "RepPP.RepPPInfo", 
                       null, 
                       null,
                       null);
            // Add to default constructor
            strBody = "            Dim app As RepPP.Application\r\n" +
                      "\r\n" +
                      "            If Not DesignMode Then\r\n" +
                      "                app = RepPP.Application.CreateFromRes()\r\n" +
                      "                app.TypedInstance.RegisterAll(True)\r\n" +
                      "                InitEditor(app, New RepPP.Toolkit.Window.UIConfigurationHelper(\"" + strProjectName + ".cfg\"), Nothing, Nothing)\r\n" +
                      "                InitSelectionBuffer(\"" + strSBCardName + "\", New RepPP.Framework.Window.SingleChildEditorFactory(Me, \"" + strCardName + "\", GetType(" + strChildFormName + ")))\r\n" +
                      "            End If";

            AddLineToFunction(strProjectName,
                              strSBFormName,
                              "New",
                              strBody,
                              -1);
	        // Add main function to the form
            AddMainFunction(strProjectName, strSBFormName);
        }

        //**********************************************************************
        //
        // Format string witch defines a property setting
        //   In: 
        //       strOwner     :    The form name
        //       strPropName  :    property name
        //       strPropType  :    property type
        //       strPropValue :    property values
        //       strRetValue  :
        //
        //   Returns:
        //       The formated string 
        //
        //**********************************************************************
        private string FormatPropLine(string strOwner, 
                                      string strPropName, 
                                      string strPropType, 
                                      string strPropValue) {
            object strRetValue;

            strRetValue = strOwner + "." + strPropName + " = ";
            if (strOwner != "Me") {
                strRetValue = "Me." + strRetValue;
            }
            switch(strPropType) {
            case "String":
                strRetValue = strRetValue + "\"" + strPropValue + "\"";
                break;
            case "Integer":
                strRetValue = strRetValue + strPropValue;
                break;
            case "Object":
                strRetValue = strRetValue + "New " + strPropValue;
                break;
            default:
                break;
            }
            return (string) strRetValue;
        }

        //**********************************************************************
        //
        // Get a property value
        //   In: 
        //       strProjectName:    The project name
        //       strFormName   :    The form in witch the control have to be added
        //       strCtlName    :    The control name
        //       strPropName   :    The property name
        //   Returns:
        //       The property value as a string
        //
        //**********************************************************************
        public override string GetProperty(string strProjectName, 
                                           string strFormName, 
                                           string strCtlName, 
                                           string strPropName) {
            string          strRetVal;
            Project         oProject;
            EditPoint       epStart;
            EditPoint       epEnd;
            ProjectItem     projItem;
            CodeClass       codeForm;
            CodeFunction    codeFunc;
            string          strTmp;
            int             iIndex;
            int             iindex2;

            strRetVal = "";
            oProject  = GetProject(strProjectName);
            projItem  = oProject.ProjectItems.Item(strFormName + ".vb");
#if VC8
            projItem = projItem.ProjectItems.Item(strFormName + ".Designer.vb");
#endif
            codeForm  = projItem.FileCodeModel.CodeElements.Item(1) as CodeClass;
            codeFunc  = (CodeFunction)codeForm.Members.Item("InitializeComponent");
            epStart   = codeFunc.StartPoint.CreateEditPoint();
            epEnd     = codeFunc.EndPoint.CreateEditPoint();
            if (strCtlName == "Me") {
                strTmp = "Me." + strPropName;
            } else {
                strTmp = "Me." + strCtlName + "." + strPropName;
            }
            if (FindText(ref epStart, epEnd, strTmp)) {
                epStart.StartOfLine();
                strTmp = epStart.GetLines(epStart.Line, epStart.Line + 1);
                iIndex = strTmp.IndexOf("=") + 1;
                if (iIndex > 0) {
                    iindex2 = strTmp.IndexOf("New");
                    if (iindex2 > 0) {
				        strRetVal = strTmp.Substring(iindex2 + 4).Trim();
                    } else {
				        strRetVal = strTmp.Substring(iIndex).Trim();
                    }
                }
            }
            return strRetVal;
        }

        //**********************************************************************
        //
        // Set a property value
        //   In: 
        //       strProjectName:    The project name
        //       strFormName   :    The form in witch the control have to be added
        //       strCtlName    :    The control name
        //       strPropName   :    The property name
        //       strPropType   :    The property type
        //       strPropValue  :    The property value
        //   Returns:
        //       
        //**********************************************************************
        public override void SetProperty(string strProjectName, 
                                         string strFormName, 
                                         string strCtlName, 
                                         string strPropName, 
                                         string strPropType, 
                                         string strPropValue) {
            Project      oProject;
            EditPoint    epStart;
            EditPoint    epEnd;
            ProjectItem  projItem;
            CodeClass    codeForm;
            CodeFunction codeFunc;
            string       strTmp;

            oProject   = GetProject(strProjectName);
            projItem   = oProject.ProjectItems.Item(strFormName + ".vb");
#if VC8
            projItem = projItem.ProjectItems.Item(strFormName + ".Designer.vb");
#endif
            codeForm   = projItem.FileCodeModel.CodeElements.Item(1) as CodeClass;
            codeFunc   = (CodeFunction)codeForm.Members.Item("InitializeComponent");
            epStart    = codeFunc.StartPoint.CreateEditPoint();
            epEnd      = codeFunc.EndPoint.CreateEditPoint();
            strCtlName = ((strCtlName == "") ? "Me" : strCtlName);
            if (strCtlName == "Me") {
                strTmp = "Me." + strPropName;
            } else {
                strTmp = "Me." + strCtlName + "." + strPropName;
            }
            if (FindText(ref epStart, epEnd, strTmp)) {
                strTmp = epStart.GetLines(epStart.Line, epStart.Line + 1);
                epStart.Delete(strTmp.Length);
                epStart.Insert(FormatPropLine(strCtlName, 
                                              strPropName, 
                                              strPropType, 
                                              strPropValue) + "\r\n" + "\t");
            }
        }

        //**********************************************************************
        //
        // Add a function header
        //   In: 
        //       ep                 :  Start edit point 
        //       strFuncName        :  Function name
        //       strFuncComment     :  Function comment
        //       strReturnComment   :  Return comment
        //       aParam             :  
        //       aParamComment      :  Params comments
        //   Returns:
        //       
        //**********************************************************************
        private void AddFunctionHeader(EditPoint ep, 
                                       string    strFuncName, 
                                       string    strFuncComment, 
                                       string    strReturnComment, 
                                       object[]  aParam, 
                                       object[]  aParamComment) {
            int iIndex;

            //ep.Insert("\t" + "'******************************************************************" + "\r\n");
            //ep.Insert("\t" + "'" + "\r\n");
            ep.Insert("\t" + "''' <summary>" + "\r\n");
            ep.Insert("\t" + "''' " + strFuncName + ":" + strFuncComment + "\r\n");
            ep.Insert("\t" + "''' </summary>" + "\r\n");
            if (aParam != null && aParamComment != null) {
                for (iIndex = 0; iIndex <= aParam.Length - 1; iIndex++) {
                    ep.Insert("\t" + "''' <param name=" + "\"" + aParam[iIndex] + "\"" + ">" +
                              aParamComment[iIndex] + "</param>" + "\r\n");
                }
            }
            if (strReturnComment != "") {
                ep.Insert("\t" + "''' <returns>" + "\r\n");
                ep.Insert("\t" + "''' " + strReturnComment + "\r\n");
                ep.Insert("\t" + "''' </returns>" + "\r\n");
            }
            //ep.Insert("\t" + "' " + "\r\n");
            //ep.Insert("\t" + "'******************************************************************" + "\r\n");
        }

        //**********************************************************************
        //
        // Add a function to a form
        //   In: 
        //       strProjectName  :    The project name
        //       strFormName     :    The form in witch the control have to be added
        //       strName         :    The function name
        //       eKind           :    kind of the function(exp: Public Shared Function)
        //       strRetType      :    Type of the return value
        //       aParamNames     :    Array of parameter names
        //       aParamTypes     :    Array of parameter types
        //       strFuncComment  :    Function comment
        //       strReturnComment:    Return comment
        //       aParamComment   :    Params comments
        //       eAccess         :
        //       strBody         :    The body of the function(optional)
        //       strCtlEvent     :    The contol to handle his event 
        //       strEvent        :    The event handled by this function(optional)
        //       strEventType    :    The event handler type
        //       bAddDelegate    :
        //   Returns:
        //       
        //**********************************************************************
        public override void AddFunction(string         strProjectName, 
                                         string         strFormName, 
                                         string         strName, 
                                         vsCMFunction   eKind, 
                                         string         strRetType, 
                                         object[]       aParamNames, 
                                         object[]       aParamTypes, 
                                         string         strFuncComment, 
                                         string         strReturnComment, 
                                         object[]       aParamComment, 
                                         vsCMAccess     eAccess, 
                                         string         strBody, 
                                         string         strCtlEvent, 
                                         string         strEvent, 
                                         string         strEventType, 
                                         bool           bAddDelegate) {
            Project        oProject;
            EditPoint      epStart;
            ProjectItem    projItem;
            CodeClass      codeForm;
            int            iIndex;
            string         strTmp = "";
            string         strTmp2;

            switch(eAccess) {
            case vsCMAccess.vsCMAccessPrivate:
                strTmp = "Private";
                break;
            case vsCMAccess.vsCMAccessPublic:
                strTmp = "Public";
                break;
            case vsCMAccess.vsCMAccessProtected:
                strTmp = "Protected";
                break;
            default:
                break;
            }
            if ((eKind & EnvDTE.vsCMFunction.vsCMFunctionVirtual) != 0) {
                strTmp = strTmp + " Overrides";
            }
            if ((eKind & vsCMFunction.vsCMFunctionSub) != 0) {
                strTmp = strTmp + " Sub";
            } else if ((eKind & vsCMFunction.vsCMFunctionShared) != 0 ){
                strTmp = " Shared Sub";
            } else {
                strTmp = strTmp + " Function";
            }
            strTmp = strTmp + " " + strName + "(";
            if (aParamNames != null) {
                strTmp = strTmp + aParamNames[0] + " As " + aParamTypes[0];
                for (iIndex = 1; iIndex <= aParamNames.Length - 1; iIndex++) {
                    strTmp = strTmp + ", " + aParamNames[iIndex] + " As " + aParamTypes[iIndex];
                }
            }
            strTmp = strTmp + ")";
            if (strRetType != "") {
                strTmp = strTmp + " As " + strRetType;
            }
            if (strEvent != "") {
                if (strCtlEvent == "") {
                    strCtlEvent = "Me";
                }
                if (!bAddDelegate) {
                    strTmp += " Handles " + strCtlEvent + "."  + strEvent;
                } else {
                    strTmp2 = "\t" + "\t" + "AddHandler " + strCtlEvent + "." + strEvent + ", AddressOf " + strName;
                    AddLineToFunction(strProjectName, 
                                      strFormName, 
                                      "New", 
                                      strTmp2, 
                                      -1);
                    if (bAddDelegate) {
                        strTmp2 = "\t" + "\t" + "\t" + "RemoveHandler " + strCtlEvent + "." + strEvent + ", AddressOf " + strName;
                        AddToDispose(strProjectName, 
                                     strFormName, 
                                     strTmp2);
                    }
                }
            }
            oProject = GetProject(strProjectName);
            projItem = oProject.ProjectItems.Item(strFormName + ".vb");
            codeForm = projItem.FileCodeModel.CodeElements.Item(1) as CodeClass;
            epStart  = codeForm.EndPoint.CreateEditPoint();
            epStart.StartOfLine();
            AddFunctionHeader(epStart, 
                              strName, 
                              strFuncComment, 
                              strReturnComment, 
                              aParamNames, 
                              aParamComment);
            epStart.Insert("\t" + strTmp + "\r\n");
            if (strBody != "") {
                epStart.Insert(strBody + "\r\n");
            }
            if ((eKind & vsCMFunction.vsCMFunctionSub)!= 0) {
                epStart.Insert("\t" + "End Sub" + "\r\n");
            }
            else if ((eKind & vsCMFunction.vsCMFunctionShared) != 0) {
                epStart.Insert("\t" + "End Sub" + "\r\n");
            } else {
                epStart.Insert("\t" + "End Function" + "\r\n");
            }
        }

        //**********************************************************************
        //
        // Add a function to a form
        //   In: 
        //       strProjectName   :  The project name
        //       strFormName      :  The form in witch the control have to be added
        //       strName          :  The function name
        //       ekind            :  Kind of the function(exp: Public Shared Function)
        //       strRetType       :  Type of the return value
        //       aParamNames      :  Array of parameter names
        //       aParamTypes      :  Array of parameter types
        //       strFuncComment   :  Function comment
        //       strReturnComment :  Return comment
        //       aParamComment    :  Params comments
        //   Returns:
        //       
        //**********************************************************************
        public override void AddFunction(string         strProjectName, 
                                         string         strFormName, 
                                         string         strName, 
                                         vsCMFunction   eKind, 
                                         string         strRetType, 
                                         object[]       aParamNames, 
                                         object[]       aParamTypes, 
                                         string         strFuncComment, 
                                         string         strReturnComment, 
                                         object[]       aParamComment) {
            AddFunction(strProjectName, 
                        strFormName, 
                        strName, 
                        eKind, 
                        strRetType, 
                        aParamNames, 
                        aParamTypes, 
                        strFuncComment, 
                        strReturnComment, 
                        aParamComment, 
                        vsCMAccess.vsCMAccessDefault, 
                        "", 
                        "", 
                        "", 
                        "System.EventHandler", 
                        false);
        }

        //**********************************************************************
        //
        // Add a function to a form
        //   In: 
        //       strProjectName  :    The project name
        //       strFormName     :    The form in witch the control have to be added
        //       strName         :    The function name
        //       eKind           :    kind of the function(exp: Public Shared Function)
        //       strRetType      :    Type of the return value
        //       aParamNames     :    Array of parameter names
        //       aParamTypes     :    Array of parameter types
        //       strFuncComment  :    Function comment
        //       strReturnComment:    Return comment
        //       aParamComment   :    Params comments
        //       eAccess         :
        //   Returns:
        //       
        //**********************************************************************
        public override void AddFunction(string         strProjectName, 
                                         string         strFormName, 
                                         string         strName, 
                                         vsCMFunction   eKind, 
                                         string         strRetType, 
                                         object[]       aParamNames, 
                                         object[]       aParamTypes, 
                                         string         strFuncComment, 
                                         string         strReturnComment, 
                                         object[]       aParamComment, 
                                         vsCMAccess     eAccess) {
            AddFunction(strProjectName, 
                        strFormName, 
                        strName, 
                        eKind, 
                        strRetType, 
                        aParamNames, 
                        aParamTypes, 
                        strFuncComment, 
                        strReturnComment, 
                        aParamComment, 
                        eAccess, 
                        "", 
                        "", 
                        "", 
                        "System.EventHandler", 
                        false);
            }

        //**********************************************************************
        //
        // Add a function to a form
        //   In: 
        //       strProjectName  :    The project name
        //       strFormName     :    The form in witch the control have to be added
        //       strName         :    The function name
        //       eKind           :    kind of the function(exp: Public Shared Function)
        //       strRetType      :    Type of the return value
        //       aParamNames     :    Array of parameter names
        //       aParamTypes     :    Array of parameter types
        //       strFuncComment  :    Function comment
        //       strReturnComment:    Return comment
        //       aParamComment   :    Params comments
        //       eAccess         :
        //       strBody         :    The body of the function(optional)
        //   Returns:
        //       
        //**********************************************************************
        public override void AddFunction(string        strProjectName, 
                                        string         strFormName, 
                                        string         strName, 
                                        vsCMFunction   eKind, 
                                        string         strRetType, 
                                        object[]       aParamNames, 
                                        object[]       aParamTypes, 
                                        string         strFuncComment, 
                                        string         strReturnComment, 
                                        object[]       aParamComment, 
                                        vsCMAccess     eAccess, 
                                        string         strBody) {
            AddFunction(strProjectName, 
                        strFormName, 
                        strName, 
                        eKind, 
                        strRetType, 
                        aParamNames, 
                        aParamTypes, 
                        strFuncComment, 
                        strReturnComment, 
                        aParamComment, 
                        eAccess, 
                        strBody, 
                        "", 
                        "", 
                        "System.EventHandler", 
                        false);
            }

        //**********************************************************************
        //
        // Add a function to a form
        //   In: 
        //       strProjectName  :    The project name
        //       strFormName     :    The form in witch the control have to be added
        //       strName         :    The function name
        //       eKind           :    kind of the function(exp: Public Shared Function)
        //       strRetType      :    Type of the return value
        //       aParamNames     :    Array of parameter names
        //       aParamTypes     :    Array of parameter types
        //       strFuncComment  :    Function comment
        //       strReturnComment:    Return comment
        //       aParamComment   :    Params comments
        //       eAccess         :
        //       strBody         :    The body of the function(optional)
        //       strCtlEvent     :    The contol to handle his event 
        //   Returns:
        //       
        //**********************************************************************
        public override void AddFunction(string         strProjectName, 
                                         string         strFormName, 
                                         string         strName, 
                                         vsCMFunction   eKind, 
                                         string         strRetType, 
                                         object[]       aParamNames, 
                                         object[]       aParamTypes, 
                                         string         strFuncComment, 
                                         string         strReturnComment, 
                                         object[]       aParamComment, 
                                         vsCMAccess     eAccess, 
                                         string         strBody, 
                                         string         strCtlEvent) {
            AddFunction(strProjectName, 
                        strFormName, 
                        strName, 
                        eKind, 
                        strRetType, 
                        aParamNames, 
                        aParamTypes, 
                        strFuncComment, 
                        strReturnComment, 
                        aParamComment, 
                        eAccess, 
                        strBody, 
                        strCtlEvent, 
                        "", 
                        "System.EventHandler", 
                        false);
            }

        //**********************************************************************
        //
        // Add a function to a form
        //   In: 
        //       strProjectName  :    The project name
        //       strFormName     :    The form in witch the control have to be added
        //       strName         :    The function name
        //       eKind           :    kind of the function(exp: Public Shared Function)
        //       strRetType      :    Type of the return value
        //       aParamNames     :    Array of parameter names
        //       aParamTypes     :    Array of parameter types
        //       strFuncComment  :    Function comment
        //       strReturnComment:    Return comment
        //       aParamComment   :    Params comments
        //       eAccess         :
        //       strBody         :    The body of the function(optional)
        //       strCtlEvent     :    The contol to handle his event 
        //       strEvent        :    The event handled by this function(optional)
        //   Returns:
        //       
        //**********************************************************************
        public override void AddFunction(string         strProjectName, 
                                         string         strFormName, 
                                         string         strName, 
                                         vsCMFunction   eKind, 
                                         string         strRetType, 
                                         object[]       aParamNames, 
                                         object[]       aParamTypes, 
                                         string         strFuncComment, 
                                         string         strReturnComment, 
                                         object[]       aParamComment, 
                                         vsCMAccess     eAccess, 
                                         string         strBody, 
                                         string         strCtlEvent, 
                                         string         strEvent) {
            AddFunction(strProjectName, 
                        strFormName, 
                        strName, 
                        eKind, 
                        strRetType, 
                        aParamNames, 
                        aParamTypes, 
                        strFuncComment, 
                        strReturnComment, 
                        aParamComment, 
                        eAccess, 
                        strBody, 
                        strCtlEvent, 
                        strEvent, 
                        "System.EventHandler", 
                        false);
            }

        //**********************************************************************
        //
        // Add a function to a form
        //   In: 
        //       strProjectName  :    The project name
        //       strFormName     :    The form in witch the control have to be added
        //       strName         :    The function name
        //       eKind           :    kind of the function(exp: Public Shared Function)
        //       strRetType      :    Type of the return value
        //       aParamNames     :    Array of parameter names
        //       aParamTypes     :    Array of parameter types
        //       strFuncComment  :    Function comment
        //       strReturnComment:    Return comment
        //       aParamComment   :    Params comments
        //       eAccess         :
        //       strBody         :    The body of the function(optional)
        //       strCtlEvent     :    The contol to handle his event 
        //       strEvent        :    The event handled by this function(optional)
        //       strEventType    :    The event handler type
        //   Returns:
        //       
        //**********************************************************************
        public override void AddFunction(string         strProjectName, 
                                         string         strFormName, 
                                         string         strName, 
                                         vsCMFunction   eKind, 
                                         string         strRetType, 
                                         object[]       aParamNames, 
                                         object[]       aParamTypes, 
                                         string         strFuncComment, 
                                         string         strReturnComment, 
                                         object[]       aParamComment, 
                                         vsCMAccess     eAccess, 
                                         string         strBody, 
                                         string         strCtlEvent, 
                                         string         strEvent, 
                                         string         strEventType) {
            AddFunction(strProjectName, 
                        strFormName, 
                        strName, 
                        eKind, 
                        strRetType, 
                        aParamNames, 
                        aParamTypes, 
                        strFuncComment, 
                        strReturnComment, 
                        aParamComment, 
                        eAccess, 
                        strBody, 
                        strCtlEvent, 
                        strEvent, 
                        strEventType, 
                        false);
            }

        //**********************************************************************
        //
        // Add a constructor to a form
        //   In: 
        //       strProjectName :    The project name
        //       strFormName    :    The form in witch the control have to be added
        //       aParamNames    :    Array of parameter names
        //       aParamTypes    :    Array of parameter types
        //       aBaseParamNames:    Array of parameter names
        //       strBody        :    The body of the function
        //       strFuncComment :    Function comment
        //       aParamComment  :    Params comments
        //   Returns:
        //       
        //**********************************************************************
        public override void AddConstructor(string      strProjectName, 
                                            string      strFormName, 
                                            object[]    aParamNames, 
                                            object[]    aParamTypes, 
                                            object[]    aBaseParamNames, 
                                            string      strBody, 
                                            string      strFuncComment, 
                                            object[]    aParamComment) {
            Project         oProject;
            EditPoint       epStart;
            ProjectItem     projItem;
            CodeClass       codeForm;
            int             iIndex;
            string          strTmp;

            oProject = GetProject(strProjectName);
            projItem = oProject.ProjectItems.Item(strFormName + ".vb");
            codeForm = projItem.FileCodeModel.CodeElements.Item(1) as CodeClass;
            epStart  = codeForm.StartPoint.CreateEditPoint();
            epStart.StartOfLine();
            epStart.LineDown(2);
            strTmp = "public Sub New(";
            if (aParamNames != null) {
                strTmp = strTmp + aParamNames[0] + " As " + aParamTypes[0];
                for (iIndex=1; iIndex <= aParamNames.Length - 1; iIndex++) {
                    strTmp = strTmp + ", " + aParamNames[iIndex] + " As " + aParamTypes[iIndex];
                }
            }
            strTmp = strTmp + ")";
            if (aBaseParamNames != null) {
                strTmp = strTmp + "\r\n" + "\t" + "\t" + "MyBase.New(" + aBaseParamNames[0];
                for(iIndex = 1; iIndex <= aBaseParamNames.Length - 1; iIndex++) {
                    strTmp = strTmp + ", " + aBaseParamNames[iIndex];
                }
                strTmp = strTmp + ")";
            }
            AddFunctionHeader(epStart, "Constructor", strFuncComment, "", aParamNames, aParamComment);
            epStart.Insert("\t" + strTmp + "\r\n");
            if (strBody != "") {
                epStart.Insert(strBody + "\r\n");
            }
            epStart.Insert("\t" + "End Sub" + "\r\n");
        }

        //**********************************************************************
        //
        // Add a line to the function
        //   In: 
        //       strProjectName:    The project name
        //       strFormName   :    The form in witch the control have to be added
        //       strFuncName   :    The function name
        //       strLine       :    The line to add
        //       iLinePos      :    The line postion (-1 at the end)
        //   Returns:
        //       
        //**********************************************************************
        public override void AddLineToFunction(string strProjectName, 
                                               string strFormName, 
                                               string strFuncName, 
                                               string strLine, 
                                               int    iLinePos) {
            Project         oProject;
            EditPoint       epStart;
            ProjectItem     projItem;
            CodeClass       codeForm;
            CodeFunction    codeFunc = null;

            oProject = GetProject(strProjectName);
            projItem = oProject.ProjectItems.Item(strFormName + ".vb");
            codeForm = projItem.FileCodeModel.CodeElements.Item(1) as CodeClass;
            foreach (CodeElement codeEl in codeForm.Members) {
                if (codeEl.Name == strFuncName) {
                    codeFunc = codeEl as CodeFunction;
                    break;
                }
            }
            if (iLinePos == - 1) {
                epStart = codeFunc.EndPoint.CreateEditPoint();
                epStart.StartOfLine();
            } else {
                epStart = codeFunc.StartPoint.CreateEditPoint();
                epStart.StartOfLine();
                epStart.LineDown(iLinePos);
            }
            epStart.Insert(strLine + "\r\n");
        }

        //**********************************************************************
        //
        // Add a line to the dispose
        //   In: 
        //       strProjectName  :    The project name
        //       strFormName     :    The form in witch the control have to be added
        //       strBody         :    Dispose code (can be empty)
        //   Returns:
        //       
        //**********************************************************************
        public override void AddToDispose(string strProjectName, 
                                          string strFormName, 
                                          string strBody) {
            Project         oProject;
            EditPoint       epStart;
            ProjectItem     projItem;
            CodeClass       codeForm;
            CodeFunction    codeFunc;

            oProject = GetProject(strProjectName);
            projItem = oProject.ProjectItems.Item(strFormName + ".vb");
            codeForm = projItem.FileCodeModel.CodeElements.Item(1) as CodeClass;
            codeFunc = (CodeFunction)codeForm.Members.Item("Dispose");
            epStart  = codeFunc.EndPoint.CreateEditPoint();
            epStart.StartOfLine();
            epStart.LineUp(2);
            epStart.Insert(strBody + "\r\n");
        }

        //**********************************************************************
        //
        // Add begin form code
        //   In: 
        //       strProjectName  :    The project name
        //       strFormName     :    The form in witch the control have to be added
        //       strBaseFormName :    The base form name
        //       strCardName     :    The card name
        //       bUseSelectBuffer:    True to indicates that a selection buffer is used
        //       strClassWrapName:    Class wrapper name for the card instance
        //   Returns:
        //       
        //**********************************************************************
        public override void AddBeginFormCode(string    strProjectName, 
                                              string    strFormName,
                                              string    strBaseForm, 
                                              string    strCardName, 
                                              bool      bUseSelectBuffer, 
                                              string    strClassWrapName) {
            string strTmp;
            string strCfgName;
            string strRootGinstName = "";

            if (!bUseSelectBuffer) {
                // Add Constructor
                strTmp = "\t" + "\t" + "Dim app As RepPP.Application" + "\r\n";
                AddLineToFunction(strProjectName, 
                                  strFormName, 
                                  "New", 
                                  strTmp, 
                                  3);
                strCfgName = "./" + strFormName + ".cfg";
                strTmp = "\t" + "\t" + "\t" + "app = RepPP.Application.Create()" + "\r\n" + "\t" + "\t" + "\t" + "app.Init(" + "\"" + m_oRepPP.SystemName + "\"" + ", " + "\"" + m_oRepPP.ProgramName + "\"" + ", " + "\"" + m_oRepPP.ConnectionName + "\"" + ", " + "\"" + m_oRepPP.UserName + "\"" + ", " + "\"" + "\"" + ", " + "RepPP.LanguageCode." + m_oRepPP.LanguageCode.ToString() + ", 0)";
                if (m_genTypedInstParam.m_eGenType != Utility.GenTypedInstTypeE.None) {
                    strTmp = strTmp + 
                             "\r\n" + "\t" + "\t" + "app.TypedInstance.RegisterAll(true)";
                }
                strTmp = strTmp +
                         "\r\n" + "\t" + "\t" + "InitV(app, " + "\"" + strCfgName + "\"" + ", " + "\"" + strFormName + "\"" + ", " + "\"" + strCardName + "\"" + ", Nothing, Nothing)";
                if (m_genTypedInstParam.m_eGenType != Utility.GenTypedInstTypeE.None) {
                    strTmp = strTmp + 
                             "\r\n" + "\t" + "\t" + "m_cinst" + strCardName + " = CType(CardFramework.CardInstance, " + strClassWrapName + ")";
                }
                AddLineToFunction(strProjectName, 
                                  strFormName, 
                                  "New", 
                                  strTmp, 
                                  -1);
                // Add InitCardHandlerV to add typed card instance 
                if (m_genTypedInstParam.m_eGenType != Utility.GenTypedInstTypeE.None) {
                    //strTmp = "\t" + "\t" + "app.TypedInstance.RegisterAll(true)" 
                    //       + "\r\n" + "\t" + "\t" + "MyBase.InitCardHandlerV(strCardName)";
                    //AddFunction(strProjectName, 
                    //            strFormName, 
                    //            "InitCardHandlerV", 
                    //          vsCMFunction.vsCMFunctionSub | vsCMFunction.vsCMFunctionVirtual, 
                    //            "", 
                    //            new object[] {"strCardName"}, new object[] {"string"}, 
                    //            "               Initialize the editor card handle framework", 
                    //            "", 
                    //            new object[] {"      Card name"}, 
                    //            vsCMAccess.vsCMAccessProtected, 
                    //            strTmp);
                }
                // Add main function to the form
                AddMainFunction(strProjectName, strFormName);
            } else {
                if (m_genTypedInstParam.m_eGenType != Utility.GenTypedInstTypeE.None) {
                    strRootGinstName = "m_cinst" + strCardName + ".gi" + m_oRepPP.Cards.Find(strCardName).ChildCardGroups[0].Name;
                } else {
                    strRootGinstName = "CardInstance.RootGroupInstance";
                }
                // Add constructor
                strTmp = "\t" + "\t" + "InitializeComponent()";
		        AddConstructor(strProjectName, 
                               strFormName,
                               new object[] {"fatherEditor", "strCardName"},
                               new object[] {"RepPP.Framework.Window.frmSBEditorBase", "string"},
                               new object[] {"fatherEditor", "strCardName", "0"},
                               strTmp,
                               "",
                               new object[] {"      Father editor", "       Name of the card attached to the child editor"});
                // Add AttachChildEditor
                strTmp = "\t" + "\t" + "MyBase.AttachChildEditor(ctlFather)";
		        if (m_genTypedInstParam.m_eGenType != Utility.GenTypedInstTypeE.None) {
			        strTmp = strTmp + "\r\n" + "\t" + "\t" + "m_cinst" + strCardName + " = CType(CardInstance, " + strClassWrapName + ")";
		        }
                AddFunction(strProjectName, 
                            strFormName,
                            "AttachChildEditor",
                            (vsCMFunction.vsCMFunctionSub | vsCMFunction.vsCMFunctionVirtual), 
                            "",
                            new object[] {"ctlFather"}, 
                            new object[] {"Control"},
                            "           Attach this child editor to the father editor",
                            "",
                            new object[] {"        Container control"},
                            vsCMAccess.vsCMAccessPublic,
                           strTmp);
                // Add SelectItem
                strTmp = "\t" + "\t" + "Dim bRetVal As Boolean\r\n" +
                         "\r\n" +
                         "\t" + "\t" + "bRetVal = MyBase.SelectItem(arrPK)\r\n" +
                         "\t" + "\t" + "If bRetVal Then\r\n" +
                         "\t" + "\t" + "    'TODO: Add user code after selectiong an item\r\n" + 
                         "\t" + "\t" + "End If\r\n" +
                         "\t" + "\t" + "Return(bRetVal)";
                AddFunction(strProjectName, 
                            strFormName,
                            "SelectItem", 
                            vsCMFunction.vsCMFunctionFunction | vsCMFunction.vsCMFunctionVirtual,
                            "Boolean",
                            new object[] {"arrPK"},
                            new object[] {"String()"},
                            "             Select an item using its primary key",
                            "True: Set openned, False: Set not openned", 
                            new object[] {"    Primary key values"},
                            vsCMAccess.vsCMAccessPublic,
                            strTmp);
                // Add InsertNewLine
                strTmp = "\t" + "\t" + "Dim bRetVal As Boolean\r\n" +
                         "\r\n" +
                         "\t" + "\t" + "bRetVal = MyBase.InsertNewLine(bAtEnd)\r\n" +
                         "\t" + "\t" + "If bRetVal Then\r\n" +
                         "\t" + "\t" + "    'TODO: Add user code after inserting a new item\r\n" + 
                         "\t" + "\t" + "End If\r\n" +
                         "\t" + "\t" + "Return(bRetVal)";
                AddFunction(strProjectName, 
                            strFormName,
                            "InsertNewLine", 
                            vsCMFunction.vsCMFunctionFunction | vsCMFunction.vsCMFunctionVirtual,
                            "Boolean",
                            new object[] {"bAtEnd"},
                            new object[] {"Boolean"},
                            "           Insert a new item",
                            "True: Item inserted, False: Error", 
                            new object[] {"    True to insert at the end of the list"},
                            vsCMAccess.vsCMAccessPublic,
                            strTmp);
                // Add DuplicateLine
                strTmp = "\t" + "\t" + "Dim bRetVal As Boolean\r\n" +
                         "\r\n" +
                         "\t" + "\t" + "bRetVal = MyBase.DuplicateLine()\r\n" +
                         "\t" + "\t" + "If bRetVal Then\r\n" +
                         "\t" + "\t" + "    'TODO: Add user code after duplicating the current item\r\n" + 
                         "\t" + "\t" + "End If\r\n" +
                         "\t" + "\t" + "Return(bRetVal)";
                AddFunction(strProjectName, 
                            strFormName,
                            "DuplicateLine", 
                            vsCMFunction.vsCMFunctionFunction | vsCMFunction.vsCMFunctionVirtual,
                            "Boolean",
                            null,
                            null,
                            "             Duplicate the current item",
                            "True: Item duplicated, False: Error", 
                            null,
                            vsCMAccess.vsCMAccessPublic,
                            strTmp);
                // Add DeleteCurrentLine
                strTmp = "\t" + "\t" + "Dim bRetVal As Boolean\r\n" +
                         "\r\n" +
                         "\t" + "\t" + "bRetVal = MyBase.DeleteCurrentLine()\r\n" +
                         "\t" + "\t" + "If bRetVal Then\r\n" +
                         "\t" + "\t" + "    'TODO: Add user code after deleting the current item\r\n" + 
                         "\t" + "\t" + "End If\r\n" +
                         "\t" + "\t" + "Return(bRetVal)";
                AddFunction(strProjectName, 
                            strFormName,
                            "DeleteCurrentLine", 
                            vsCMFunction.vsCMFunctionFunction | vsCMFunction.vsCMFunctionVirtual,
                            "Boolean",
                            null,
                            null,
                            "             Delete the current item",
                            "True: Item deleted, False: Error", 
                            null,
                            vsCMAccess.vsCMAccessPublic,
                            strTmp);
                // Add SaveLine
                strTmp = "\t" + "\t" + "Dim bRetVal As Boolean\r\n" +
                         "\r\n" +
                         "\t" + "\t" + "'TODO: Add user code before saving the current item\r\n" +
                         "\t" + "\t" + "bRetVal = MyBase.SaveLine()\r\n" +
                         "\t" + "\t" + "If bRetVal Then\r\n" +
                         "\t" + "\t" + "    'TODO: Add user code after saving the current item\r\n" + 
                         "\t" + "\t" + "End If\r\n" +
                         "\t" + "\t" + "Return(bRetVal)";
                AddFunction(strProjectName, 
                            strFormName,
                            "SaveLine", 
                            vsCMFunction.vsCMFunctionFunction | vsCMFunction.vsCMFunctionVirtual,
                            "Boolean",
                            null,
                            null,
                            "             Save the current item",
                            "True: Item saved, False: Error", 
                            null,
                            vsCMAccess.vsCMAccessPublic,
                            strTmp);
            }
            // Add typed card instance member
            if (m_genTypedInstParam.m_eGenType != Utility.GenTypedInstTypeE.None) {
                AddVariable(strProjectName, 
                            strFormName, 
                            "m_cinst" + strCardName, 
                            strClassWrapName, 
                            vsCMAccess.vsCMAccessPrivate, 
                            strCardName + " card instance",
                            false,
                            null);
            }
        }

        //**********************************************************************
        //
        // Add code related to card group control initialization / clean up 
        //   In: 
        //       strProjectName     :    The project name
        //       strFormName        :    The form in witch the control have to be added
        //       strGrpCtlName      :
        //       strGrpName         :    The group name
        //       strFatherCrdGrpCtl :    The father card group control name
        //       strGridName        :    The grid name
        //       bPopupChildForm    :    Indicates if the current form is a popup child one
        //       iChildIndex        :    Child index in the card
        //       bUseSelectBuffer   :    Indicates if the project uses a selection buffer
        //   Returns:
        //       
        //**********************************************************************
        public override void AddCardGroupInitCode(string strProjectName, 
                                                  string strFormName, 
                                                  string strGrpCtlName, 
                                                  string strGrpName, 
                                                  string strFatherCrdGrpCtl, 
                                                  string strGridName, 
                                                  bool   bPopupChildForm, 
                                                  int    iChildIndex, 
                                                  bool   bUseSelectBuffer) {
	        string strTmp;
	        string strTmp2;
	        string strCrdGrpName;

	        strCrdGrpName = "m_cgrpHandler" + strGrpName;
		    AddVariable(strProjectName, 
                        strFormName, 
                        strCrdGrpName, 
                        "RepPP.Framework.Window.CardGroupHandler", 
                        vsCMAccess.vsCMAccessPrivate, 
                        strGrpName + " card group manager object",
                        false,
                        null);
	        if (strFatherCrdGrpCtl == "") {
		        strTmp = "\t" + "\t" + strCrdGrpName + " = Handler" + "\r\n" + 
                         "\t" + "\t" + strGrpCtlName + ".CardGroupHandler = " + strCrdGrpName;
	        } else {
                strTmp2 = "\t" + "\t" + "\t" + "If Not " + strCrdGrpName + " Is Nothing Then" + "\r\n" + "\t" + "\t" + "\t" + "\t" + strCrdGrpName + ".Dispose()" + "\r\n" + "\t" + "\t" + "\t" + "\t" + strCrdGrpName + " = Nothing" + "\r\n" + "\t" + "\t" + "\t" + "End If";
                AddToDispose(strProjectName, 
                             strFormName, 
                             strTmp2);
                strTmp = "\t" + "\t" + strCrdGrpName + " = new RepPP.Framework.Window.CardGroupHandler(" + strFatherCrdGrpCtl + ", CardGroup.Card.FindCardGroup(" + "\"" + strGrpName + "\"" + "), true)" + "\r\n" + "\t" + "\t" + strGrpCtlName + ".CardGroupHandler = " + strCrdGrpName;
	        }
	        if (strGridName != "") {
		        strTmp = strTmp + "\r\n" + "\t" + "\t" + "\t" + strCrdGrpName + ".LineListHandler = " + strGridName;
	        }
            if (bPopupChildForm) {
	            AddLineToFunction(strProjectName, 
                                  strFormName, 
                                  "OnLoad", 
                                  strTmp, 
                                  - 1);
            } else {
	            AddLineToFunction(strProjectName, 
                                  strFormName, 
                                  "AttachChildEditor", 
                                  strTmp, 
                                  - 1);
            }
        }

        //**********************************************************************
        //
        // Add code related to card group control initialization / clean up 
        //   In: 
        //       strProjectName:    The project name
        //       strFormName   :    The form in witch the control have to be added
        //       strGrpCtlName :
        //       strGrpName    :    The group name
        //   Returns:
        //       
        //**********************************************************************
        public override void AddCardGroupCleanCode(string strProjectName, 
                                                   string strFormName, 
                                                   string strGrpCtlName, 
                                                   string strGrpName) {
            string strtmp;

            strtmp = "\t" + "\t" + "If e.Cancel = False Then" + "\r\n" + "\t" + "\t" + "\t" + "e.Cancel = Not Me." + strGrpCtlName + ".ValidateCurLine()" + "\r\n" + "\t" + "\t" + "End If";
            AddLineToFunction(strProjectName, 
                              strFormName, 
                              "OnFormClosing", 
                              strtmp, 
                              -1);
        }

        //**********************************************************************
        //
        // Add a Begin child form code
        //   In: 
        //       strProjectName  :    The project name
        //       strFormName     :    The form in witch the control have to be added
        //       strBaseFormName :    The base form name
        //       arrCardName     :    Array of childs card name
        //       bUseSelectBuffer:    True to use a selection buffer
        //   Returns:
        //       
        //**********************************************************************
        public override void AddBeginChildCode(string   strProjectName, 
                                               string   strFormName, 
                                               string   strBaseFormName, 
                                               string[] arrCrdGrpName, 
                                               bool     bUseSelectBuffer) {


	        string  strBody;
	        string  strCrdGrpList;

	        // Add Constructor
	        //strCrdGrpList    = BuildStringFromArray(arrCardName);
	        //strCrdGrpList    = "new string[] {" + strCardsList + "}";
            strCrdGrpList   = "crdGrpFather.CardGroup.Card.FindCardGroup(\"" + arrCrdGrpName[0] + "\")";
	        strBody         = "\t" + "\t" + "InitializeComponent()";
  	        AddConstructor(strProjectName, 
                           strFormName, 
                           new object[] {"fatherEditor", "crdGrpFather"}, 
                           new object[] {"RepPP.Framework.Window.frmSBEditorBase", "RepPP.Framework.Window.CardGroupHandler"}, 
                           new object[] {"fatherEditor", "crdGrpFather", strCrdGrpList, "true"},
                           strBody,
                           "", 
                           new object[] {"     Top editor form", "  Father card group handler"});
            AddFunction(strProjectName, 
                        strFormName,
                        "OnLoad",
                        (vsCMFunction.vsCMFunctionSub | vsCMFunction.vsCMFunctionVirtual),
                        "",
                        new object[] {"e"}, 
                        new object[] {"System.EventArgs"},
                        "           Raise the load event",
                        "",
                        new object[] {"        Event arguments"},
                        vsCMAccess.vsCMAccessProtected,
                        "MyBase.OnLoad(e)");
        }

        //**********************************************************************
        //
        // Add End child form code
        //   In: 
        //       strProjectName :    The project name
        //       strFormName    :    The form in witch the control have to be added
        //   Returns:
        //       
        //**********************************************************************
        public override void AddEndChildCode(string strProjectName, 
                                             string strFormName, 
                                             bool   bUseSelectBuffer) {
//            AddLineToFunction(strProjectName, 
//                              strFormName, 
//                              "New", 
//                              "\t" + "\t" + "CardFramework.Refresh()", 
//                              -1);
        }

        //**********************************************************************
        //
        // Get unique name for the control by adding an index
        //   In: 
        //       strProjectName:    The project name
        //       strFormName   :    The form in witch the control have to be added
        //       strCtlName    :    The control name
        //   Returns:
        //       The new control name
        //
        //**********************************************************************
        protected override string GetUniqueCtlName(string strProjectName, 
                                                   string strFormName, 
                                                   string strCtlName) {
            string          strRetVal;
            Project         oProject;
            ProjectItem     projItem;
            CodeClass       codeForm;
            int             iIndex;
            bool            bNameOk;

            strRetVal = AdjustComponentName(strCtlName, 
                                            "Control1");
            oProject  = GetProject(strProjectName);
            projItem  = oProject.ProjectItems.Item(strFormName + ".vb");
#if VC8
            projItem = projItem.ProjectItems.Item(strFormName + ".Designer.vb");
#endif
            codeForm  = projItem.FileCodeModel.CodeElements.Item(1) as CodeClass;
            bNameOk   = false;
            iIndex    = 1;
            while (!bNameOk) {
                bNameOk = true;
                foreach (CodeElement codeElem in codeForm.Members) {
			        if (codeElem.Name == strRetVal) {
                        bNameOk = false;
                        break;
                    }
                }
                if (!bNameOk) {
                    strRetVal = strCtlName + iIndex;
                    iIndex = iIndex + 1;
                }
            }
            return strRetVal;
        }

        //**********************************************************************
        //
        // Get unique name for the control by adding an index
        //   In: 
        //       strProjectName:    The project name
        //       strFormName   :    The form in witch the control have to be added
        //       strCtlName    :    The control name
        //   Returns:
        //       The new control name
        //
        //**********************************************************************
        protected override string GetUniqueFormName(string strProjectName, 
                                                    string strFormName) {
            string      strRetVal;
            Project     oProject;
            int         iIndex;
            bool        bNameOk;

            strRetVal = AdjustComponentName(strFormName, "Form1");
            oProject  = GetProject(strProjectName);
            bNameOk   = false;
            iIndex    = 1;
            while (!bNameOk) {
                bNameOk = true;
                foreach (ProjectItem projItem in oProject.ProjectItems) {
                    if (projItem.Name == strRetVal + ".vb") {
                        bNameOk = false;
                        break;
                    }
                }
                if (!bNameOk) {
                    strRetVal = strFormName + iIndex;
                    iIndex = iIndex + 1;
                }
            }
            return strRetVal;
        }

        //**********************************************************************
        //
        // Add a code to call child form from the parent form
        //   In: 
        //       strProjectName   :    The project name
        //       strFormName      :    The parent form name
        //       strChildName     :    The child  name
        //       strBtnName       :    The button control name
        //       strCrdgrpFather  :    The father card group control name
        //                             it's used to initialize the child card group control
        //       bUseSelectBuffer :
        //       bInMainForm      :
        //   Returns:
        //       The necessary height to design the Group fields in the form 
        //
        //**********************************************************************
        public override void AddCallToChildForm(string strProjectName, 
                                                string strFormName, 
                                                string strChildName, 
                                                string strBtnName, 
                                                string strCrdgrpFather, 
                                                bool   bUseSelectBuffer, 
                                                bool   bInMainForm) {
            string strTmp;

            strTmp = "\t" + "\t" + "Using frm as New " + strChildName + "(FatherEditor, " + strCrdgrpFather + ")" + "\r\n" + 
                     "\t" + "\t" + "\t" + "frm.ShowDialog()" + "\r\n" + 
                     "\t" + "\t" + "End Using";
            AddFunction(strProjectName, 
                        strFormName, 
                        strBtnName + "_Click", 
                        vsCMFunction.vsCMFunctionSub, 
                        "", 
                        new object[] {"sender", "e"}, 
                        new object[] {"Object", "EventArgs"}, 
                        "        Handle the " + strBtnName + " button click event", 
                        "", 
                        new object[] {"  Event sender", "       Event arguments"}, 
                        vsCMAccess.vsCMAccessPrivate, 
                        strTmp, 
                        strBtnName, 
                        "Click");
        }

        //**********************************************************************
        //
        // Add a variable to a form
        //   In: 
        //       strProjectName:    The project name
        //       strFormName   :    The form in witch the variable have to be added
        //       strVarName    :    The variable name
        //       strVarType    :    The variable type
        //       eAccess       :    The variable access type
        //       bIsConstant   :    If the variable is constant
        //       strInitValue  :    Initialization value
        //   Returns:
        //
        //**********************************************************************
        protected override void AddVariable(string      strProjectName, 
                                            string      strFormName, 
                                            string      strVarName, 
                                            string      strVarType, 
                                            vsCMAccess  eAccess, 
                                            string      strComment,
                                            bool        bIsConstant,
                                            string      strInitValue) {
	        Project      oProject;
	        ProjectItem  projItem;
	        CodeClass    codeForm;
	        CodeVariable codeVar;

	        oProject                = GetProject(strProjectName);
	        projItem                = oProject.ProjectItems.Item(strFormName + ".vb");
            codeForm                = projItem.FileCodeModel.CodeElements.Item(1) as CodeClass;
	        codeVar                 = codeForm.AddVariable(strVarName, strVarType, 0, eAccess, null);
            // Not implemented
            // codeVar.InitExpression  = strInitValue;            
            // codeVar.IsConstant      = bIsConstant;
	        // codeVar.Comment         = strComment;
        }

        //*********************************************************     
        //
        /// <summary>
        /// GenCardTypedInstance:           Genearte card typed instance 
        /// </summary>
        /// <param name="strPrjName">       Project name</param>
        /// <param name="strCardName">      Card name</param>
        //
        //*********************************************************     
        protected override void GenCardTypedInstance(string strPrjName, string strCardName) {
            Utility.GenCardTypedInstance(Utility.GetProject(m_vsApp, strPrjName),
                                         m_genTypedInstParam.m_strCardInstClassPrefix,
                                         m_genTypedInstParam.m_strGroupInstClassPrefix,
                                         m_genTypedInstParam.m_strGroupInstPropPrefix,
                                         m_genTypedInstParam.m_strFieldPropPrefix,
                                         m_genTypedInstParam.m_strNamespace,
                                         m_genTypedInstParam.m_strRootNamespace,
                                         m_genTypedInstParam.m_strCardInstBaseClass,
                                         m_genTypedInstParam.m_strGroupInstBaseClass,
                                         new TypedInstanceGenForVB(),
                                         (m_genTypedInstParam.m_eGenType == Utility.GenTypedInstTypeE.TypedInstAndWSAdapter),
                                         m_oRepPP.Cards[strCardName],
                                         Utility.ReplaceFileModeE.sdReplaceWithoutAsk,
                                         String.Empty,
                                         true);
        }

        //*********************************************************     
        //
        /// <summary>
        /// GenGroupTypedInstance:      Generate group typed instance 
        /// </summary>
        /// <param name="strPrjName">   Project name</param>
        /// <param name="strCardName">  Group name</param>
        //
        //*********************************************************     
        protected override void GenGroupTypedInstance(string strPrjName, string strGroupName) {
            Utility.GenGroupTypedInstance(Utility.GetProject(m_vsApp, strPrjName),
                                          m_genTypedInstParam.m_strCardInstClassPrefix,
                                          m_genTypedInstParam.m_strGroupInstClassPrefix,
                                          m_genTypedInstParam.m_strGroupInstPropPrefix,
                                          m_genTypedInstParam.m_strFieldPropPrefix,
                                          m_genTypedInstParam.m_strNamespace,
                                          m_genTypedInstParam.m_strRootNamespace,
                                          m_genTypedInstParam.m_strGroupInstBaseClass,
                                          new TypedInstanceGenForVB(),
                                          (m_genTypedInstParam.m_eGenType == Utility.GenTypedInstTypeE.TypedInstAndWSAdapter),
                                          m_oRepPP.Groups[strGroupName],
                                          Utility.ReplaceFileModeE.sdReplaceWithoutAsk,
                                          String.Empty);
        }

        //**********************************************************************
        //
        /// <summary>
        /// SetTypedInstNamespace:          Set the namespace for the typed instances
        /// </summary>
        /// <param name="strProjectName">   Project Name</param>
        //
        //**********************************************************************
        protected override void SetTypedInstNamespace(string strProjectName) {
            if (m_genTypedInstParam.m_strNamespace == String.Empty &&
                m_genTypedInstParam.m_strRootNamespace == String.Empty) {
                m_genTypedInstParam.m_strRootNamespace = Utility.GetProjectRootNamespace(m_vsApp, strProjectName);
                if (m_genTypedInstParam.m_strRootNamespace == String.Empty) {
                    m_genTypedInstParam.m_strNamespace = strProjectName;
                }
            }
        }
    }
}
