//*********************************************************
//
// frmStepModeSelect:    Step form to select the generation mode (Application, item)
//                       and to select the card or the group in the repository
//
// Author:		         Maher Najah
//
// (c) 2003 by Consyst SQL Inc.
//
//
//*********************************************************
using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Xml;
using sdfwwebcsnet;
using RepPP.VSNetWizard;

namespace RepPP.VSNetWizard
{
	/// <summary>
	/// frmStepModeSelect: Select card or group
	/// </summary>
	internal class frmStepModeSelect : frmStepBase {
        protected System.Windows.Forms.Label            lblSBCard;
        protected System.Windows.Forms.ComboBox         cboSBCard;
        protected System.Windows.Forms.GroupBox         grpGenType;
        protected System.Windows.Forms.RadioButton      radCard;
        protected System.Windows.Forms.RadioButton      radGroup;
        protected System.Windows.Forms.ComboBox         cboGroupOrCard;
        protected System.Windows.Forms.Label            lblGroupOrCard;
        protected System.Windows.Forms.CheckBox         chkUseSB;
        protected System.Windows.Forms.RadioButton      radApplication;
		private System.ComponentModel.Container         components = null;

        //******************************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        //
        //******************************************************************
		public frmStepModeSelect() {
			InitializeComponent();
		}

        //******************************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        /// <param name="frmMain">      Main form</param>
        //
        //******************************************************************
		public frmStepModeSelect(frmMainBase frmMain) : base(frmMain) {
			InitializeComponent();
            if (frmMain.WizardMode == GenerationModeE.sdApplication) {
                radApplication.Checked = true;
                radCard.Enabled        = false;
                radGroup.Enabled       = false;
            } else {
                radApplication.Enabled = false;
                radCard.Checked        = true;
            }
            chkUseSB.Checked                     = true;
            cboGroupOrCard.SelectedIndexChanged += new System.EventHandler(OnGroupSelectedIndexChanged);
            cboSBCard.SelectedIndexChanged      += new System.EventHandler(OnSBCardSelectedIndexChanged);
            radCard.CheckedChanged              += new System.EventHandler(OnCardCheckedChanged);
            chkUseSB.CheckedChanged             += new System.EventHandler(OnUseSBCheckedChanged);
		}

        //******************************************************************
        //
        /// <summary>
        /// Dispose:                    Dispose the form
        /// </summary>
        /// <param name="bDisposing">   true if disposing, false if finalizing</param>
        //
        //******************************************************************
        protected override void Dispose(bool bDisposing) {
            if (bDisposing) {
                if (components != null) {
                    components.Dispose();
                }
            }
            base.Dispose(bDisposing);
        }

        //******************************************************************
        //
        /// <summary>
        /// InitV:                    Initialize the step
        /// </summary>
        //
        //******************************************************************
        public override void InitV() {
            if (radGroup.Checked) {
                lblGroupOrCard.Text = "Group";
            } else {
                lblGroupOrCard.Text = "Transaction Card";
            }
            cboGroupOrCard.Items.Clear();
            cboSBCard.Items.Clear();
            if (radGroup.Checked) {
                foreach(RepPP.Group grp in m_frmMain.Application.Groups) {
                    cboGroupOrCard.Items.Add(grp.Name);
                }
            } else {
                foreach(RepPP.Card crd in m_frmMain.Application.Cards) {
                    cboGroupOrCard.Items.Add(crd.Name);
                    if (cboSBCard.Enabled) {
                        cboSBCard.Items.Add(crd.Name);
                    }
                }
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// ValidateB:                    Validate the step
        /// </summary>
        /// <returns>
        /// True:   Valid
        /// False:  Not Valid
        /// </returns>
        //
        //******************************************************************
        public override bool ValidateB() {
            bool bRetVal;

            if (radGroup.Checked) {
                bRetVal = (cboGroupOrCard.Text != String.Empty);
            } else {
                bRetVal = (cboGroupOrCard.Text != String.Empty                               && 
                          (chkUseSB.Checked == false || cboSBCard.Text != String.Empty));
            }
            return(bRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// GroupName:             Group name
        /// </summary>
        //
        //******************************************************************
        public string GroupName {
            get {
                string strRetVal = null;

                if (radGroup.Checked) {
                    strRetVal = cboGroupOrCard.Text;
                }
                return(strRetVal);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// CardName:             Card name
        /// </summary>
        //
        //******************************************************************
        public string CardName {
            get {
                string strRetVal = null;

                if (radCard.Checked || radApplication.Checked) {
                    strRetVal = cboGroupOrCard.Text;
                }
                return(strRetVal);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// SBCardName:             Selection buffer card name
        /// </summary>
        //
        //******************************************************************
        public string SBCardName {
            get {
                string strRetVal = null;

                if (cboSBCard.Enabled) {
                    strRetVal = cboSBCard.Text;
                }
                return(strRetVal);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// GenerationMode:     Generation maode(0: group, 1: card, 2: card with selection buffer)
        /// </summary>
        //
        //******************************************************************
        public GenerationModeE GenerationMode {
            get {
                GenerationModeE eRetVal;

                if (radApplication.Checked) {
                    eRetVal = GenerationModeE.sdApplication;
                } else if (radGroup.Checked) {
                    eRetVal = GenerationModeE.sdItemGroup;
                } else {
                    eRetVal = GenerationModeE.sdItemCard;
                }
                return(eRetVal);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnGroupSelectedIndexChanged:    Handle group combobox selected index changed
        /// </summary>
        /// <param name="sender">           Sender object   </param>
        /// <param name="e">                Event Arguments </param>
        //  
        //*********************************************************     
        private void OnGroupSelectedIndexChanged(object sender, System.EventArgs e) {
            m_frmMain.NextButton.Enabled = ValidateB();
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnSBCardSelectedIndexChanged:    Handle SB card combobox selected index changed
        /// </summary>
        /// <param name="sender">            Sender object   </param>
        /// <param name="e">                 Event Arguments </param>
        //  
        //*********************************************************     
        private void OnSBCardSelectedIndexChanged(object sender, System.EventArgs e) {
            m_frmMain.NextButton.Enabled = ValidateB();
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnCardCheckedChanged:   Handle card radio button click event
        /// </summary>
        /// <param name="sender">   Sender object   </param>
        /// <param name="e">        Event Arguments </param>
        //  
        //*********************************************************     
        private void OnCardCheckedChanged(object sender, System.EventArgs e) {
            chkUseSB.Enabled             = radCard.Checked;
            cboSBCard.Enabled            = radCard.Checked;
            m_frmMain.NextButton.Enabled = ValidateB();
            InitV();
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnUseSBCheckedChanged:   Handle use SB check button click event
        /// </summary>
        /// <param name="sender">    Sender object   </param>
        /// <param name="e">         Event Arguments </param>
        //  
        //*********************************************************     
        private void OnUseSBCheckedChanged(object sender, System.EventArgs e) {
            m_frmMain.NextButton.Enabled = ValidateB();
            cboSBCard.Enabled            = chkUseSB.Checked;
        }
		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            this.cboGroupOrCard = new System.Windows.Forms.ComboBox();
            this.lblGroupOrCard = new System.Windows.Forms.Label();
            this.cboSBCard = new System.Windows.Forms.ComboBox();
            this.lblSBCard = new System.Windows.Forms.Label();
            this.grpGenType = new System.Windows.Forms.GroupBox();
            this.radApplication = new System.Windows.Forms.RadioButton();
            this.radCard = new System.Windows.Forms.RadioButton();
            this.radGroup = new System.Windows.Forms.RadioButton();
            this.chkUseSB = new System.Windows.Forms.CheckBox();
            this.pnlStep.SuspendLayout();
            this.grpGenType.SuspendLayout();
            this.SuspendLayout();
            // 
            // pnlStep
            // 
            this.pnlStep.Controls.Add(this.chkUseSB);
            this.pnlStep.Controls.Add(this.grpGenType);
            this.pnlStep.Controls.Add(this.cboSBCard);
            this.pnlStep.Controls.Add(this.lblSBCard);
            this.pnlStep.Controls.Add(this.cboGroupOrCard);
            this.pnlStep.Controls.Add(this.lblGroupOrCard);
            this.pnlStep.Name = "pnlStep";
            this.pnlStep.Size = new System.Drawing.Size(416, 280);
            // 
            // cboGroupOrCard
            // 
            this.cboGroupOrCard.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
            this.cboGroupOrCard.Location = new System.Drawing.Point(136, 160);
            this.cboGroupOrCard.Name = "cboGroupOrCard";
            this.cboGroupOrCard.Size = new System.Drawing.Size(256, 21);
            this.cboGroupOrCard.Sorted = true;
            this.cboGroupOrCard.TabIndex = 2;
            // 
            // lblGroupOrCard
            // 
            this.lblGroupOrCard.Location = new System.Drawing.Point(16, 160);
            this.lblGroupOrCard.Name = "lblGroupOrCard";
            this.lblGroupOrCard.Size = new System.Drawing.Size(120, 24);
            this.lblGroupOrCard.TabIndex = 2;
            this.lblGroupOrCard.Text = "Group";
            // 
            // cboSBCard
            // 
            this.cboSBCard.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
            this.cboSBCard.Location = new System.Drawing.Point(136, 192);
            this.cboSBCard.Name = "cboSBCard";
            this.cboSBCard.Size = new System.Drawing.Size(256, 21);
            this.cboSBCard.Sorted = true;
            this.cboSBCard.TabIndex = 3;
            // 
            // lblSBCard
            // 
            this.lblSBCard.Location = new System.Drawing.Point(16, 192);
            this.lblSBCard.Name = "lblSBCard";
            this.lblSBCard.Size = new System.Drawing.Size(112, 24);
            this.lblSBCard.TabIndex = 4;
            this.lblSBCard.Text = "Selection Buffer Card";
            // 
            // grpGenType
            // 
            this.grpGenType.Controls.Add(this.radApplication);
            this.grpGenType.Controls.Add(this.radCard);
            this.grpGenType.Controls.Add(this.radGroup);
            this.grpGenType.Location = new System.Drawing.Point(16, 16);
            this.grpGenType.Name = "grpGenType";
            this.grpGenType.Size = new System.Drawing.Size(376, 104);
            this.grpGenType.TabIndex = 0;
            this.grpGenType.TabStop = false;
            this.grpGenType.Text = "Generation Type";
            // 
            // radApplication
            // 
            this.radApplication.Location = new System.Drawing.Point(24, 24);
            this.radApplication.Name = "radApplication";
            this.radApplication.Size = new System.Drawing.Size(128, 16);
            this.radApplication.TabIndex = 0;
            this.radApplication.Text = "Application";
            // 
            // radCard
            // 
            this.radCard.Location = new System.Drawing.Point(24, 72);
            this.radCard.Name = "radCard";
            this.radCard.Size = new System.Drawing.Size(128, 16);
            this.radCard.TabIndex = 2;
            this.radCard.Text = "Card Forms";
            // 
            // radGroup
            // 
            this.radGroup.Location = new System.Drawing.Point(24, 48);
            this.radGroup.Name = "radGroup";
            this.radGroup.Size = new System.Drawing.Size(128, 16);
            this.radGroup.TabIndex = 1;
            this.radGroup.Text = "Group  Form";
            // 
            // chkUseSB
            // 
            this.chkUseSB.Location = new System.Drawing.Point(136, 132);
            this.chkUseSB.Name = "chkUseSB";
            this.chkUseSB.Size = new System.Drawing.Size(144, 16);
            this.chkUseSB.TabIndex = 1;
            this.chkUseSB.Text = "Use Selection Buffer";
            // 
            // frmStepModeSelect
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(448, 302);
            this.Name = "frmStepModeSelect";
            this.Text = "frmStepModeSelect";
            this.pnlStep.ResumeLayout(false);
            this.grpGenType.ResumeLayout(false);
            this.ResumeLayout(false);

        }
		#endregion
	} // Class frmStepModeSelect
} // Namespace